/*-*- mode:c;indent-tabs-mode:nil;c-basic-offset:2;tab-width:8;coding:utf-8 -*-│
│vi: set net ft=c ts=2 sts=2 sw=2 fenc=utf-8                                :vi│
╞══════════════════════════════════════════════════════════════════════════════╡
│ Copyright 2021 Justine Alexandra Roberts Tunney                              │
│                                                                              │
│ Permission to use, copy, modify, and/or distribute this software for         │
│ any purpose with or without fee is hereby granted, provided that the         │
│ above copyright notice and this permission notice appear in all copies.      │
│                                                                              │
│ THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL                │
│ WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED                │
│ WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE             │
│ AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL         │
│ DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR        │
│ PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER               │
│ TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR             │
│ PERFORMANCE OF THIS SOFTWARE.                                                │
╚─────────────────────────────────────────────────────────────────────────────*/
#include "libc/macros.internal.h"
#include "libc/str/str.h"
#include "libc/dce.h"

static const char16_t kUpperToLower[][2] = {
    {u'À', u'à'},      // 00c0 → 00e0
    {u'Á', u'á'},      // 00c1 → 00e1
    {u'Â', u'â'},      // 00c2 → 00e2
    {u'Ã', u'ã'},      // 00c3 → 00e3
    {u'Ä', u'ä'},      // 00c4 → 00e4
    {u'Å', u'å'},      // 00c5 → 00e5
    {u'Æ', u'æ'},      // 00c6 → 00e6
    {u'Ç', u'ç'},      // 00c7 → 00e7
    {u'È', u'è'},      // 00c8 → 00e8
    {u'É', u'é'},      // 00c9 → 00e9
    {u'Ê', u'ê'},      // 00ca → 00ea
    {u'Ë', u'ë'},      // 00cb → 00eb
    {u'Ì', u'ì'},      // 00cc → 00ec
    {u'Í', u'í'},      // 00cd → 00ed
    {u'Î', u'î'},      // 00ce → 00ee
    {u'Ï', u'ï'},      // 00cf → 00ef
    {u'Ð', u'ð'},      // 00d0 → 00f0
    {u'Ñ', u'ñ'},      // 00d1 → 00f1
    {u'Ò', u'ò'},      // 00d2 → 00f2
    {u'Ó', u'ó'},      // 00d3 → 00f3
    {u'Ô', u'ô'},      // 00d4 → 00f4
    {u'Õ', u'õ'},      // 00d5 → 00f5
    {u'Ö', u'ö'},      // 00d6 → 00f6
    {u'Ø', u'ø'},      // 00d8 → 00f8
    {u'Ù', u'ù'},      // 00d9 → 00f9
    {u'Ú', u'ú'},      // 00da → 00fa
    {u'Û', u'û'},      // 00db → 00fb
    {u'Ü', u'ü'},      // 00dc → 00fc
    {u'Ý', u'ý'},      // 00dd → 00fd
    {u'Þ', u'þ'},      // 00de → 00fe
#ifndef TINY
    {u'Ā', u'ā'},      // 0100 → 0101
    {u'Ă', u'ă'},      // 0102 → 0103
    {u'Ą', u'ą'},      // 0104 → 0105
    {u'Ć', u'ć'},      // 0106 → 0107
    {u'Ĉ', u'ĉ'},      // 0108 → 0109
    {u'Ċ', u'ċ'},      // 010a → 010b
    {u'Č', u'č'},      // 010c → 010d
    {u'Ď', u'ď'},      // 010e → 010f
    {u'Đ', u'đ'},      // 0110 → 0111
    {u'Ē', u'ē'},      // 0112 → 0113
    {u'Ĕ', u'ĕ'},      // 0114 → 0115
    {u'Ė', u'ė'},      // 0116 → 0117
    {u'Ę', u'ę'},      // 0118 → 0119
    {u'Ě', u'ě'},      // 011a → 011b
    {u'Ĝ', u'ĝ'},      // 011c → 011d
    {u'Ğ', u'ğ'},      // 011e → 011f
    {u'Ġ', u'ġ'},      // 0120 → 0121
    {u'Ģ', u'ģ'},      // 0122 → 0123
    {u'Ĥ', u'ĥ'},      // 0124 → 0125
    {u'Ħ', u'ħ'},      // 0126 → 0127
    {u'Ĩ', u'ĩ'},      // 0128 → 0129
    {u'Ī', u'ī'},      // 012a → 012b
    {u'Ĭ', u'ĭ'},      // 012c → 012d
    {u'Į', u'į'},      // 012e → 012f
    {u'Ĳ', u'ĳ'},      // 0132 → 0133
    {u'Ĵ', u'ĵ'},      // 0134 → 0135
    {u'Ķ', u'ķ'},      // 0136 → 0137
    {u'Ĺ', u'ĺ'},      // 0139 → 013a
    {u'Ļ', u'ļ'},      // 013b → 013c
    {u'Ľ', u'ľ'},      // 013d → 013e
    {u'Ŀ', u'ŀ'},      // 013f → 0140
    {u'Ł', u'ł'},      // 0141 → 0142
    {u'Ń', u'ń'},      // 0143 → 0144
    {u'Ņ', u'ņ'},      // 0145 → 0146
    {u'Ň', u'ň'},      // 0147 → 0148
    {u'Ŋ', u'ŋ'},      // 014a → 014b
    {u'Ō', u'ō'},      // 014c → 014d
    {u'Ŏ', u'ŏ'},      // 014e → 014f
    {u'Ő', u'ő'},      // 0150 → 0151
    {u'Œ', u'œ'},      // 0152 → 0153
    {u'Ŕ', u'ŕ'},      // 0154 → 0155
    {u'Ŗ', u'ŗ'},      // 0156 → 0157
    {u'Ř', u'ř'},      // 0158 → 0159
    {u'Ś', u'ś'},      // 015a → 015b
    {u'Ŝ', u'ŝ'},      // 015c → 015d
    {u'Ş', u'ş'},      // 015e → 015f
    {u'Š', u'š'},      // 0160 → 0161
    {u'Ţ', u'ţ'},      // 0162 → 0163
    {u'Ť', u'ť'},      // 0164 → 0165
    {u'Ŧ', u'ŧ'},      // 0166 → 0167
    {u'Ũ', u'ũ'},      // 0168 → 0169
    {u'Ū', u'ū'},      // 016a → 016b
    {u'Ŭ', u'ŭ'},      // 016c → 016d
    {u'Ů', u'ů'},      // 016e → 016f
    {u'Ű', u'ű'},      // 0170 → 0171
    {u'Ų', u'ų'},      // 0172 → 0173
    {u'Ŵ', u'ŵ'},      // 0174 → 0175
    {u'Ŷ', u'ŷ'},      // 0176 → 0177
    {u'Ÿ', u'ÿ'},      // 0178 → 00ff
    {u'Ź', u'ź'},      // 0179 → 017a
    {u'Ż', u'ż'},      // 017b → 017c
    {u'Ž', u'ž'},      // 017d → 017e
    {u'Ɓ', u'ɓ'},      // 0181 → 0253
    {u'Ƃ', u'ƃ'},      // 0182 → 0183
    {u'Ƅ', u'ƅ'},      // 0184 → 0185
    {u'Ɔ', u'ɔ'},      // 0186 → 0254
    {u'Ƈ', u'ƈ'},      // 0187 → 0188
    {u'Ɗ', u'ɗ'},      // 018a → 0257
    {u'Ƌ', u'ƌ'},      // 018b → 018c
    {u'Ǝ', u'ɘ'},      // 018e → 0258
    {u'Ə', u'ə'},      // 018f → 0259
    {u'Ɛ', u'ɛ'},      // 0190 → 025b
    {u'Ƒ', u'ƒ'},      // 0191 → 0192
    {u'Ɠ', u'ɠ'},      // 0193 → 0260
    {u'Ɣ', u'ɣ'},      // 0194 → 0263
    {u'Ɩ', u'ɩ'},      // 0196 → 0269
    {u'Ɨ', u'ɨ'},      // 0197 → 0268
    {u'Ƙ', u'ƙ'},      // 0198 → 0199
    {u'Ɯ', u'ɯ'},      // 019c → 026f
    {u'Ɲ', u'ɲ'},      // 019d → 0272
    {u'Ơ', u'ơ'},      // 01a0 → 01a1
    {u'Ƣ', u'ƣ'},      // 01a2 → 01a3
    {u'Ƥ', u'ƥ'},      // 01a4 → 01a5
    {u'Ƨ', u'ƨ'},      // 01a7 → 01a8
    {u'Ʃ', u'ʃ'},      // 01a9 → 0283
    {u'Ƭ', u'ƭ'},      // 01ac → 01ad
    {u'Ʈ', u'ʈ'},      // 01ae → 0288
    {u'Ư', u'ư'},      // 01af → 01b0
    {u'Ʊ', u'ʊ'},      // 01b1 → 028a
    {u'Ʋ', u'ʋ'},      // 01b2 → 028b
    {u'Ƴ', u'ƴ'},      // 01b3 → 01b4
    {u'Ƶ', u'ƶ'},      // 01b5 → 01b6
    {u'Ʒ', u'ƹ'},      // 01b7 → 01b9
    {u'Ƹ', u'ƹ'},      // 01b8 → 01b9
    {u'Ƽ', u'ƽ'},      // 01bc → 01bd
    {u'Ǆ', u'ǆ'},      // 01c4 → 01c6
    {u'Ǉ', u'ǉ'},      // 01c7 → 01c9
    {u'Ǌ', u'ǌ'},      // 01ca → 01cc
    {u'Ǎ', u'ǎ'},      // 01cd → 01ce
    {u'Ǐ', u'ǐ'},      // 01cf → 01d0
    {u'Ǒ', u'ǒ'},      // 01d1 → 01d2
    {u'Ǔ', u'ǔ'},      // 01d3 → 01d4
    {u'Ǖ', u'ǖ'},      // 01d5 → 01d6
    {u'Ǘ', u'ǘ'},      // 01d7 → 01d8
    {u'Ǚ', u'ǚ'},      // 01d9 → 01da
    {u'Ǜ', u'ǜ'},      // 01db → 01dc
    {u'Ǟ', u'ǟ'},      // 01de → 01df
    {u'Ǡ', u'ǡ'},      // 01e0 → 01e1
    {u'Ǣ', u'ǣ'},      // 01e2 → 01e3
    {u'Ǥ', u'ǥ'},      // 01e4 → 01e5
    {u'Ǧ', u'ǧ'},      // 01e6 → 01e7
    {u'Ǩ', u'ǩ'},      // 01e8 → 01e9
    {u'Ǫ', u'ǫ'},      // 01ea → 01eb
    {u'Ǭ', u'ǭ'},      // 01ec → 01ed
    {u'Ǯ', u'ǯ'},      // 01ee → 01ef
    {u'Ǳ', u'ǆ'},      // 01f1 → 01c6
    {u'Ǵ', u'ǵ'},      // 01f4 → 01f5
    {u'Ǹ', u'ǹ'},      // 01f8 → 01f9
    {u'Ǻ', u'ǻ'},      // 01fa → 01fb
    {u'Ǽ', u'ǽ'},      // 01fc → 01fd
    {u'Ǿ', u'ǿ'},      // 01fe → 01ff
    {u'Ȁ', u'ȁ'},      // 0200 → 0201
    {u'Ȃ', u'ȃ'},      // 0202 → 0203
    {u'Ȅ', u'ȅ'},      // 0204 → 0205
    {u'Ȇ', u'ȇ'},      // 0206 → 0207
    {u'Ȉ', u'ȉ'},      // 0208 → 0209
    {u'Ȋ', u'ȋ'},      // 020a → 020b
    {u'Ȍ', u'ȍ'},      // 020c → 020d
    {u'Ȏ', u'ȏ'},      // 020e → 020f
    {u'Ȑ', u'ȑ'},      // 0210 → 0211
    {u'Ȓ', u'ȓ'},      // 0212 → 0213
    {u'Ȕ', u'ȕ'},      // 0214 → 0215
    {u'Ȗ', u'ȗ'},      // 0216 → 0217
    {u'Ș', u'ș'},      // 0218 → 0219
    {u'Ț', u'ț'},      // 021a → 021b
    {u'Ȝ', u'ȝ'},      // 021c → 021d
    {u'Ȟ', u'ȟ'},      // 021e → 021f
    {u'Ƞ', u'ƞ'},      // 0220 → 019e
    {u'Ȣ', u'ȣ'},      // 0222 → 0223
    {u'Ȥ', u'ȥ'},      // 0224 → 0225
    {u'Ȧ', u'ǡ'},      // 0226 → 01e1
    {u'Ȩ', u'ȩ'},      // 0228 → 0229
    {u'Ȫ', u'ȫ'},      // 022a → 022b
    {u'Ȭ', u'ȭ'},      // 022c → 022d
    {u'Ȯ', u'ȯ'},      // 022e → 022f
    {u'Ȱ', u'ȱ'},      // 0230 → 0231
    {u'Ȳ', u'ȳ'},      // 0232 → 0233
    {u'Ⱥ', u'ⱥ'},      // 023a → 2c65
    {u'Ȼ', u'ȼ'},      // 023b → 023c
    {u'Ƚ', u'ƚ'},      // 023d → 019a
    {u'Ⱦ', u'ⱦ'},      // 023e → 2c66
    {u'Ɂ', u'ɂ'},      // 0241 → 0242
    {u'Ƀ', u'ƀ'},      // 0243 → 0180
    {u'Ʉ', u'ʉ'},      // 0244 → 0289
    {u'Ʌ', u'ʌ'},      // 0245 → 028c
    {u'Ɇ', u'ɇ'},      // 0246 → 0247
    {u'Ɉ', u'ɉ'},      // 0248 → 0249
    {u'Ɍ', u'ɍ'},      // 024c → 024d
    {u'Ɏ', u'ɏ'},      // 024e → 024f
    {u'Ͱ', u'ͱ'},      // 0370 → 0371
    {u'Ͳ', u'ͳ'},      // 0372 → 0373
    {u'Ͷ', u'ͷ'},      // 0376 → 0377
    {u'Ά', u'ά'},      // 0386 → 03ac
    {u'Έ', u'έ'},      // 0388 → 03ad
    {u'Ή', u'ή'},      // 0389 → 03ae
    {u'Ί', u'ί'},      // 038a → 03af
    {u'Ό', u'ό'},      // 038c → 03cc
    {u'Ύ', u'ύ'},      // 038e → 03cd
    {u'Ώ', u'ώ'},      // 038f → 03ce
    {u'Α', u'ά'},      // 0391 → 03ac
    {u'Β', u'β'},      // 0392 → 03b2
    {u'Γ', u'γ'},      // 0393 → 03b3
    {u'Δ', u'δ'},      // 0394 → 03b4
    {u'Ε', u'έ'},      // 0395 → 03ad
    {u'Ζ', u'ζ'},      // 0396 → 03b6
    {u'Η', u'ή'},      // 0397 → 03ae
    {u'Θ', u'θ'},      // 0398 → 03b8
    {u'Ι', u'ΐ'},      // 0399 → 0390
    {u'Κ', u'κ'},      // 039a → 03ba
    {u'Λ', u'λ'},      // 039b → 03bb
    {u'Μ', u'μ'},      // 039c → 03bc
    {u'Ν', u'ν'},      // 039d → 03bd
    {u'Ξ', u'ξ'},      // 039e → 03be
    {u'Ο', u'ο'},      // 039f → 03bf
    {u'Π', u'π'},      // 03a0 → 03c0
    {u'Ρ', u'ρ'},      // 03a1 → 03c1
    {u'Σ', u'σ'},      // 03a3 → 03c3
    {u'Τ', u'τ'},      // 03a4 → 03c4
    {u'Υ', u'ΰ'},      // 03a5 → 03b0
    {u'Φ', u'φ'},      // 03a6 → 03c6
    {u'Χ', u'χ'},      // 03a7 → 03c7
    {u'Ψ', u'ψ'},      // 03a8 → 03c8
    {u'Ω', u'ω'},      // 03a9 → 03c9
    {u'Ϊ', u'ΐ'},      // 03aa → 0390
    {u'Ϋ', u'ΰ'},      // 03ab → 03b0
    {u'Ϣ', u'ϣ'},      // 03e2 → 03e3
    {u'Ϥ', u'ϥ'},      // 03e4 → 03e5
    {u'Ϧ', u'ϧ'},      // 03e6 → 03e7
    {u'Ϩ', u'ϩ'},      // 03e8 → 03e9
    {u'Ϫ', u'ϫ'},      // 03ea → 03eb
    {u'Ϭ', u'ϭ'},      // 03ec → 03ed
    {u'Ϯ', u'ϯ'},      // 03ee → 03ef
    {u'Ϸ', u'ϸ'},      // 03f7 → 03f8
    {u'Ϻ', u'ϻ'},      // 03fa → 03fb
    {u'Ͻ', u'ͻ'},      // 03fd → 037b
    {u'Ͼ', u'ͼ'},      // 03fe → 037c
    {u'Ͽ', u'ͽ'},      // 03ff → 037d
    {u'Ѐ', u'ѐ'},      // 0400 → 0450
    {u'Ё', u'ё'},      // 0401 → 0451
    {u'Ђ', u'ђ'},      // 0402 → 0452
    {u'Ѓ', u'ѓ'},      // 0403 → 0453
    {u'Є', u'є'},      // 0404 → 0454
    {u'Ѕ', u'ѕ'},      // 0405 → 0455
    {u'І', u'і'},      // 0406 → 0456
    {u'Ї', u'ї'},      // 0407 → 0457
    {u'Ј', u'ј'},      // 0408 → 0458
    {u'Љ', u'љ'},      // 0409 → 0459
    {u'Њ', u'њ'},      // 040a → 045a
    {u'Ћ', u'ћ'},      // 040b → 045b
    {u'Ќ', u'ќ'},      // 040c → 045c
    {u'Ѝ', u'ѝ'},      // 040d → 045d
    {u'Ў', u'ў'},      // 040e → 045e
    {u'Џ', u'џ'},      // 040f → 045f
    {u'А', u'а'},      // 0410 → 0430
    {u'Б', u'б'},      // 0411 → 0431
    {u'В', u'в'},      // 0412 → 0432
    {u'Г', u'г'},      // 0413 → 0433
    {u'Д', u'д'},      // 0414 → 0434
    {u'Е', u'е'},      // 0415 → 0435
    {u'Ж', u'ж'},      // 0416 → 0436
    {u'З', u'з'},      // 0417 → 0437
    {u'И', u'е'},      // 0418 → 0435
    {u'Й', u'й'},      // 0419 → 0439
    {u'К', u'к'},      // 041a → 043a
    {u'Л', u'л'},      // 041b → 043b
    {u'М', u'м'},      // 041c → 043c
    {u'Н', u'н'},      // 041d → 043d
    {u'О', u'о'},      // 041e → 043e
    {u'П', u'п'},      // 041f → 043f
    {u'Р', u'р'},      // 0420 → 0440
    {u'С', u'с'},      // 0421 → 0441
    {u'Т', u'т'},      // 0422 → 0442
    {u'У', u'у'},      // 0423 → 0443
    {u'Ф', u'ф'},      // 0424 → 0444
    {u'Х', u'х'},      // 0425 → 0445
    {u'Ц', u'ц'},      // 0426 → 0446
    {u'Ч', u'ч'},      // 0427 → 0447
    {u'Ш', u'ш'},      // 0428 → 0448
    {u'Щ', u'щ'},      // 0429 → 0449
    {u'Ъ', u'ъ'},      // 042a → 044a
    {u'Ы', u'ы'},      // 042b → 044b
    {u'Ь', u'ь'},      // 042c → 044c
    {u'Э', u'л'},      // 042d → 043b
    {u'Ю', u'ю'},      // 042e → 044e
    {u'Я', u'я'},      // 042f → 044f
    {u'Ѡ', u'ѡ'},      // 0460 → 0461
    {u'Ѣ', u'ѣ'},      // 0462 → 0463
    {u'Ѥ', u'ѥ'},      // 0464 → 0465
    {u'Ѧ', u'ѧ'},      // 0466 → 0467
    {u'Ѩ', u'ѩ'},      // 0468 → 0469
    {u'Ѫ', u'ѫ'},      // 046a → 046b
    {u'Ѭ', u'ѭ'},      // 046c → 046d
    {u'Ѯ', u'ѯ'},      // 046e → 046f
    {u'Ѱ', u'ѱ'},      // 0470 → 0471
    {u'Ѳ', u'ѳ'},      // 0472 → 0473
    {u'Ѵ', u'ѵ'},      // 0474 → 0475
    {u'Ѷ', u'ѷ'},      // 0476 → 0477
    {u'Ѹ', u'є'},      // 0478 → 0454
    {u'Ѻ', u'ѻ'},      // 047a → 047b
    {u'Ѽ', u'ѽ'},      // 047c → 047d
    {u'Ѿ', u'ѿ'},      // 047e → 047f
    {u'Ҁ', u'ҁ'},      // 0480 → 0481
    {u'Ҋ', u'ҋ'},      // 048a → 048b
    {u'Ҍ', u'ҍ'},      // 048c → 048d
    {u'Ҏ', u'ҏ'},      // 048e → 048f
    {u'Ґ', u'ґ'},      // 0490 → 0491
    {u'Ғ', u'ғ'},      // 0492 → 0493
    {u'Ҕ', u'ҕ'},      // 0494 → 0495
    {u'Җ', u'җ'},      // 0496 → 0497
    {u'Ҙ', u'ҙ'},      // 0498 → 0499
    {u'Қ', u'қ'},      // 049a → 049b
    {u'Ҝ', u'ҝ'},      // 049c → 049d
    {u'Ҟ', u'ҟ'},      // 049e → 049f
    {u'Ҡ', u'ҡ'},      // 04a0 → 04a1
    {u'Ң', u'ң'},      // 04a2 → 04a3
    {u'Ҥ', u'ҥ'},      // 04a4 → 04a5
    {u'Ҧ', u'ҧ'},      // 04a6 → 04a7
    {u'Ҩ', u'ҩ'},      // 04a8 → 04a9
    {u'Ҫ', u'ҫ'},      // 04aa → 04ab
    {u'Ҭ', u'ҭ'},      // 04ac → 04ad
    {u'Ү', u'ү'},      // 04ae → 04af
    {u'Ұ', u'ұ'},      // 04b0 → 04b1
    {u'Ҳ', u'ҳ'},      // 04b2 → 04b3
    {u'Ҵ', u'ҵ'},      // 04b4 → 04b5
    {u'Ҷ', u'ҷ'},      // 04b6 → 04b7
    {u'Ҹ', u'ҹ'},      // 04b8 → 04b9
    {u'Һ', u'һ'},      // 04ba → 04bb
    {u'Ҽ', u'ҽ'},      // 04bc → 04bd
    {u'Ҿ', u'ҿ'},      // 04be → 04bf
    {u'Ӂ', u'ӂ'},      // 04c1 → 04c2
    {u'Ӄ', u'ӄ'},      // 04c3 → 04c4
    {u'Ӆ', u'ӆ'},      // 04c5 → 04c6
    {u'Ӈ', u'ӈ'},      // 04c7 → 04c8
    {u'Ӊ', u'ӊ'},      // 04c9 → 04ca
    {u'Ӌ', u'ӌ'},      // 04cb → 04cc
    {u'Ӎ', u'ӎ'},      // 04cd → 04ce
    {u'Ӑ', u'ӑ'},      // 04d0 → 04d1
    {u'Ӓ', u'ӓ'},      // 04d2 → 04d3
    {u'Ӕ', u'ӕ'},      // 04d4 → 04d5
    {u'Ӗ', u'ӗ'},      // 04d6 → 04d7
    {u'Ә', u'ә'},      // 04d8 → 04d9
    {u'Ӛ', u'ӛ'},      // 04da → 04db
    {u'Ӝ', u'ӝ'},      // 04dc → 04dd
    {u'Ӟ', u'ӟ'},      // 04de → 04df
    {u'Ӡ', u'ӡ'},      // 04e0 → 04e1
    {u'Ӣ', u'ӣ'},      // 04e2 → 04e3
    {u'Ӥ', u'ӥ'},      // 04e4 → 04e5
    {u'Ӧ', u'ӧ'},      // 04e6 → 04e7
    {u'Ө', u'ө'},      // 04e8 → 04e9
    {u'Ӫ', u'ӫ'},      // 04ea → 04eb
    {u'Ӭ', u'ӭ'},      // 04ec → 04ed
    {u'Ӯ', u'ӯ'},      // 04ee → 04ef
    {u'Ӱ', u'ӱ'},      // 04f0 → 04f1
    {u'Ӳ', u'ӳ'},      // 04f2 → 04f3
    {u'Ӵ', u'ӵ'},      // 04f4 → 04f5
    {u'Ӷ', u'ӷ'},      // 04f6 → 04f7
    {u'Ӹ', u'ӹ'},      // 04f8 → 04f9
    {u'Ӻ', u'ӻ'},      // 04fa → 04fb
    {u'Ӽ', u'ӽ'},      // 04fc → 04fd
    {u'Ӿ', u'ӿ'},      // 04fe → 04ff
    {u'Ԁ', u'ԁ'},      // 0500 → 0501
    {u'Ԃ', u'ԃ'},      // 0502 → 0503
    {u'Ԅ', u'ԅ'},      // 0504 → 0505
    {u'Ԇ', u'ԇ'},      // 0506 → 0507
    {u'Ԉ', u'ԉ'},      // 0508 → 0509
    {u'Ԋ', u'ԋ'},      // 050a → 050b
    {u'Ԍ', u'ԍ'},      // 050c → 050d
    {u'Ԏ', u'ԏ'},      // 050e → 050f
    {u'Ԑ', u'ԑ'},      // 0510 → 0511
    {u'Ԓ', u'ԓ'},      // 0512 → 0513
    {u'Ԕ', u'ԕ'},      // 0514 → 0515
    {u'Ԗ', u'ԗ'},      // 0516 → 0517
    {u'Ԙ', u'ԙ'},      // 0518 → 0519
    {u'Ԛ', u'ԛ'},      // 051a → 051b
    {u'Ԝ', u'ԝ'},      // 051c → 051d
    {u'Ԟ', u'ԟ'},      // 051e → 051f
    {u'Ԡ', u'ԡ'},      // 0520 → 0521
    {u'Ԣ', u'ԣ'},      // 0522 → 0523
    {u'Ԥ', u'ԥ'},      // 0524 → 0525
    {u'Ԧ', u'ԧ'},      // 0526 → 0527
    {u'Ԩ', u'ԩ'},    // 0528 → 0529
    {u'Ԫ', u'ԫ'},    // 052a → 052b
    {u'Ԭ', u'ԭ'},    // 052c → 052d
    {u'Ԯ', u'ԯ'},    // 052e → 052f
    {u'Ա', u'ա'},      // 0531 → 0561
    {u'Բ', u'բ'},      // 0532 → 0562
    {u'Գ', u'գ'},      // 0533 → 0563
    {u'Դ', u'դ'},      // 0534 → 0564
    {u'Ե', u'ե'},      // 0535 → 0565
    {u'Զ', u'զ'},      // 0536 → 0566
    {u'Է', u'է'},      // 0537 → 0567
    {u'Ը', u'ը'},      // 0538 → 0568
    {u'Թ', u'թ'},      // 0539 → 0569
    {u'Ժ', u'ժ'},      // 053a → 056a
    {u'Ի', u'ի'},      // 053b → 056b
    {u'Լ', u'լ'},      // 053c → 056c
    {u'Խ', u'խ'},      // 053d → 056d
    {u'Ծ', u'ծ'},      // 053e → 056e
    {u'Կ', u'կ'},      // 053f → 056f
    {u'Հ', u'հ'},      // 0540 → 0570
    {u'Ձ', u'ձ'},      // 0541 → 0571
    {u'Ղ', u'ղ'},      // 0542 → 0572
    {u'Ճ', u'ճ'},      // 0543 → 0573
    {u'Մ', u'մ'},      // 0544 → 0574
    {u'Յ', u'յ'},      // 0545 → 0575
    {u'Ն', u'ն'},      // 0546 → 0576
    {u'Շ', u'շ'},      // 0547 → 0577
    {u'Ո', u'ո'},      // 0548 → 0578
    {u'Չ', u'չ'},      // 0549 → 0579
    {u'Պ', u'պ'},      // 054a → 057a
    {u'Ջ', u'ջ'},      // 054b → 057b
    {u'Ռ', u'ռ'},      // 054c → 057c
    {u'Ս', u'ս'},      // 054d → 057d
    {u'Վ', u'վ'},      // 054e → 057e
    {u'Տ', u'տ'},      // 054f → 057f
    {u'Ր', u'ր'},      // 0550 → 0580
    {u'Ց', u'ց'},      // 0551 → 0581
    {u'Ւ', u'ւ'},      // 0552 → 0582
    {u'Փ', u'փ'},      // 0553 → 0583
    {u'Ք', u'ք'},      // 0554 → 0584
    {u'Օ', u'օ'},      // 0555 → 0585
    {u'Ֆ', u'ֆ'},      // 0556 → 0586
    {u'Ⴀ', u'ⴀ'},      // 10a0 → 2d00
    {u'Ⴁ', u'ⴁ'},      // 10a1 → 2d01
    {u'Ⴂ', u'ⴂ'},      // 10a2 → 2d02
    {u'Ⴃ', u'ⴃ'},      // 10a3 → 2d03
    {u'Ⴄ', u'ⴄ'},      // 10a4 → 2d04
    {u'Ⴅ', u'ⴅ'},      // 10a5 → 2d05
    {u'Ⴆ', u'ⴆ'},      // 10a6 → 2d06
    {u'Ⴇ', u'ⴇ'},      // 10a7 → 2d07
    {u'Ⴈ', u'ⴈ'},      // 10a8 → 2d08
    {u'Ⴉ', u'ⴉ'},      // 10a9 → 2d09
    {u'Ⴊ', u'ⴊ'},      // 10aa → 2d0a
    {u'Ⴋ', u'ⴋ'},      // 10ab → 2d0b
    {u'Ⴌ', u'ⴌ'},      // 10ac → 2d0c
    {u'Ⴍ', u'ⴍ'},      // 10ad → 2d0d
    {u'Ⴎ', u'ⴎ'},      // 10ae → 2d0e
    {u'Ⴏ', u'ⴏ'},      // 10af → 2d0f
    {u'Ⴐ', u'ⴐ'},      // 10b0 → 2d10
    {u'Ⴑ', u'ⴑ'},      // 10b1 → 2d11
    {u'Ⴒ', u'ⴒ'},      // 10b2 → 2d12
    {u'Ⴓ', u'ⴓ'},      // 10b3 → 2d13
    {u'Ⴔ', u'ⴔ'},      // 10b4 → 2d14
    {u'Ⴕ', u'ⴕ'},      // 10b5 → 2d15
    {u'Ⴖ', u'ⴖ'},      // 10b6 → 2d16
    {u'Ⴗ', u'ⴗ'},      // 10b7 → 2d17
    {u'Ⴘ', u'ⴘ'},      // 10b8 → 2d18
    {u'Ⴙ', u'ⴙ'},      // 10b9 → 2d19
    {u'Ⴚ', u'ⴚ'},      // 10ba → 2d1a
    {u'Ⴛ', u'ⴛ'},      // 10bb → 2d1b
    {u'Ⴜ', u'ⴜ'},      // 10bc → 2d1c
    {u'Ⴝ', u'ⴝ'},      // 10bd → 2d1d
    {u'Ⴞ', u'ⴞ'},      // 10be → 2d1e
    {u'Ⴟ', u'ⴟ'},      // 10bf → 2d1f
    {u'Ⴠ', u'ⴠ'},      // 10c0 → 2d20
    {u'Ⴡ', u'ⴡ'},      // 10c1 → 2d21
    {u'Ⴢ', u'ⴢ'},      // 10c2 → 2d22
    {u'Ⴣ', u'ⴣ'},      // 10c3 → 2d23
    {u'Ⴤ', u'ⴤ'},      // 10c4 → 2d24
    {u'Ⴥ', u'ⴥ'},      // 10c5 → 2d25
    {u'Ⴧ', u'ⴧ'},      // 10c7 → 2d27
    {u'Ⴭ', u'ⴭ'},      // 10cd → 2d2d
    {u'Ᏸ', u'ᏸ'},    // 13f0 → 13f8
    {u'Ᏹ', u'ᏹ'},    // 13f1 → 13f9
    {u'Ᏺ', u'ᏺ'},    // 13f2 → 13fa
    {u'Ᏻ', u'ᏻ'},    // 13f3 → 13fb
    {u'Ᏼ', u'ᏼ'},    // 13f4 → 13fc
    {u'Ᏽ', u'ᏽ'},  // 13f5 → 13fd
    {u'Ḁ', u'ḁ'},      // 1e00 → 1e01
    {u'Ḃ', u'ḃ'},      // 1e02 → 1e03
    {u'Ḅ', u'ḅ'},      // 1e04 → 1e05
    {u'Ḇ', u'ḇ'},      // 1e06 → 1e07
    {u'Ḉ', u'ḉ'},      // 1e08 → 1e09
    {u'Ḋ', u'ḋ'},      // 1e0a → 1e0b
    {u'Ḍ', u'ḍ'},      // 1e0c → 1e0d
    {u'Ḏ', u'ḏ'},      // 1e0e → 1e0f
    {u'Ḑ', u'ḑ'},      // 1e10 → 1e11
    {u'Ḓ', u'ḓ'},      // 1e12 → 1e13
    {u'Ḕ', u'ḕ'},      // 1e14 → 1e15
    {u'Ḗ', u'ḗ'},      // 1e16 → 1e17
    {u'Ḙ', u'ḙ'},      // 1e18 → 1e19
    {u'Ḛ', u'ḛ'},      // 1e1a → 1e1b
    {u'Ḝ', u'ḝ'},      // 1e1c → 1e1d
    {u'Ḟ', u'ḟ'},      // 1e1e → 1e1f
    {u'Ḡ', u'ḡ'},      // 1e20 → 1e21
    {u'Ḣ', u'ḣ'},      // 1e22 → 1e23
    {u'Ḥ', u'ḥ'},      // 1e24 → 1e25
    {u'Ḧ', u'ḧ'},      // 1e26 → 1e27
    {u'Ḩ', u'ḩ'},      // 1e28 → 1e29
    {u'Ḫ', u'ḫ'},      // 1e2a → 1e2b
    {u'Ḭ', u'ḭ'},      // 1e2c → 1e2d
    {u'Ḯ', u'ḯ'},      // 1e2e → 1e2f
    {u'Ḱ', u'ḱ'},      // 1e30 → 1e31
    {u'Ḳ', u'ḳ'},      // 1e32 → 1e33
    {u'Ḵ', u'ḵ'},      // 1e34 → 1e35
    {u'Ḷ', u'ḷ'},      // 1e36 → 1e37
    {u'Ḹ', u'ḹ'},      // 1e38 → 1e39
    {u'Ḻ', u'ḻ'},      // 1e3a → 1e3b
    {u'Ḽ', u'ḽ'},      // 1e3c → 1e3d
    {u'Ḿ', u'ḿ'},      // 1e3e → 1e3f
    {u'Ṁ', u'ṁ'},      // 1e40 → 1e41
    {u'Ṃ', u'ṃ'},      // 1e42 → 1e43
    {u'Ṅ', u'ṅ'},      // 1e44 → 1e45
    {u'Ṇ', u'ṇ'},      // 1e46 → 1e47
    {u'Ṉ', u'ṉ'},      // 1e48 → 1e49
    {u'Ṋ', u'ṋ'},      // 1e4a → 1e4b
    {u'Ṍ', u'ṍ'},      // 1e4c → 1e4d
    {u'Ṏ', u'ṏ'},      // 1e4e → 1e4f
    {u'Ṑ', u'ṑ'},      // 1e50 → 1e51
    {u'Ṓ', u'ṓ'},      // 1e52 → 1e53
    {u'Ṕ', u'ṕ'},      // 1e54 → 1e55
    {u'Ṗ', u'ṗ'},      // 1e56 → 1e57
    {u'Ṙ', u'ṙ'},      // 1e58 → 1e59
    {u'Ṛ', u'ṛ'},      // 1e5a → 1e5b
    {u'Ṝ', u'ṝ'},      // 1e5c → 1e5d
    {u'Ṟ', u'ṟ'},      // 1e5e → 1e5f
    {u'Ṡ', u'ṡ'},      // 1e60 → 1e61
    {u'Ṣ', u'ṣ'},      // 1e62 → 1e63
    {u'Ṥ', u'ṥ'},      // 1e64 → 1e65
    {u'Ṧ', u'ṧ'},      // 1e66 → 1e67
    {u'Ṩ', u'ṩ'},      // 1e68 → 1e69
    {u'Ṫ', u'ṫ'},      // 1e6a → 1e6b
    {u'Ṭ', u'ṭ'},      // 1e6c → 1e6d
    {u'Ṯ', u'ṯ'},      // 1e6e → 1e6f
    {u'Ṱ', u'ṱ'},      // 1e70 → 1e71
    {u'Ṳ', u'ṳ'},      // 1e72 → 1e73
    {u'Ṵ', u'ṵ'},      // 1e74 → 1e75
    {u'Ṷ', u'ṷ'},      // 1e76 → 1e77
    {u'Ṹ', u'ṹ'},      // 1e78 → 1e79
    {u'Ṻ', u'ṻ'},      // 1e7a → 1e7b
    {u'Ṽ', u'ṽ'},      // 1e7c → 1e7d
    {u'Ṿ', u'ṿ'},      // 1e7e → 1e7f
    {u'Ẁ', u'ẁ'},      // 1e80 → 1e81
    {u'Ẃ', u'ẃ'},      // 1e82 → 1e83
    {u'Ẅ', u'ẅ'},      // 1e84 → 1e85
    {u'Ẇ', u'ẇ'},      // 1e86 → 1e87
    {u'Ẉ', u'ẉ'},      // 1e88 → 1e89
    {u'Ẋ', u'ẋ'},      // 1e8a → 1e8b
    {u'Ẍ', u'ẍ'},      // 1e8c → 1e8d
    {u'Ẏ', u'ẏ'},      // 1e8e → 1e8f
    {u'Ẑ', u'ẑ'},      // 1e90 → 1e91
    {u'Ẓ', u'ẓ'},      // 1e92 → 1e93
    {u'Ẕ', u'ẕ'},      // 1e94 → 1e95
    {u'ẞ', u'ß'},      // 1e9e → 00df
    {u'Ạ', u'ạ'},      // 1ea0 → 1ea1
    {u'Ả', u'ả'},      // 1ea2 → 1ea3
    {u'Ấ', u'ấ'},      // 1ea4 → 1ea5
    {u'Ầ', u'ầ'},      // 1ea6 → 1ea7
    {u'Ẩ', u'ẩ'},      // 1ea8 → 1ea9
    {u'Ẫ', u'ẫ'},      // 1eaa → 1eab
    {u'Ậ', u'ậ'},      // 1eac → 1ead
    {u'Ắ', u'ắ'},      // 1eae → 1eaf
    {u'Ằ', u'ằ'},      // 1eb0 → 1eb1
    {u'Ẳ', u'ẳ'},      // 1eb2 → 1eb3
    {u'Ẵ', u'ẵ'},      // 1eb4 → 1eb5
    {u'Ặ', u'ặ'},      // 1eb6 → 1eb7
    {u'Ẹ', u'ẹ'},      // 1eb8 → 1eb9
    {u'Ẻ', u'ẻ'},      // 1eba → 1ebb
    {u'Ẽ', u'ḛ'},      // 1ebc → 1e1b
    {u'Ế', u'ế'},      // 1ebe → 1ebf
    {u'Ề', u'ề'},      // 1ec0 → 1ec1
    {u'Ể', u'ể'},      // 1ec2 → 1ec3
    {u'Ễ', u'ễ'},      // 1ec4 → 1ec5
    {u'Ệ', u'ệ'},      // 1ec6 → 1ec7
    {u'Ỉ', u'ỉ'},      // 1ec8 → 1ec9
    {u'Ị', u'ị'},      // 1eca → 1ecb
    {u'Ọ', u'ọ'},      // 1ecc → 1ecd
    {u'Ỏ', u'ỏ'},      // 1ece → 1ecf
    {u'Ố', u'ố'},      // 1ed0 → 1ed1
    {u'Ồ', u'ồ'},      // 1ed2 → 1ed3
    {u'Ổ', u'ổ'},      // 1ed4 → 1ed5
    {u'Ỗ', u'ỗ'},      // 1ed6 → 1ed7
    {u'Ộ', u'ộ'},      // 1ed8 → 1ed9
    {u'Ớ', u'ớ'},      // 1eda → 1edb
    {u'Ờ', u'ờ'},      // 1edc → 1edd
    {u'Ở', u'ở'},      // 1ede → 1edf
    {u'Ỡ', u'ỡ'},      // 1ee0 → 1ee1
    {u'Ợ', u'ợ'},      // 1ee2 → 1ee3
    {u'Ụ', u'ụ'},      // 1ee4 → 1ee5
    {u'Ủ', u'ủ'},      // 1ee6 → 1ee7
    {u'Ứ', u'ứ'},      // 1ee8 → 1ee9
    {u'Ừ', u'ừ'},      // 1eea → 1eeb
    {u'Ử', u'ử'},      // 1eec → 1eed
    {u'Ữ', u'ữ'},      // 1eee → 1eef
    {u'Ự', u'ự'},      // 1ef0 → 1ef1
    {u'Ỳ', u'ỳ'},      // 1ef2 → 1ef3
    {u'Ỵ', u'ỵ'},      // 1ef4 → 1ef5
    {u'Ỷ', u'ỷ'},      // 1ef6 → 1ef7
    {u'Ỹ', u'ỹ'},      // 1ef8 → 1ef9
    {u'Ỻ', u'ỻ'},      // 1efa → 1efb
    {u'Ỽ', u'ỽ'},      // 1efc → 1efd
    {u'Ỿ', u'ỿ'},      // 1efe → 1eff
    {u'Ἀ', u'ἀ'},      // 1f08 → 1f00
    {u'Ἁ', u'ἁ'},      // 1f09 → 1f01
    {u'Ἂ', u'ἂ'},      // 1f0a → 1f02
    {u'Ἃ', u'ἃ'},      // 1f0b → 1f03
    {u'Ἄ', u'ἄ'},      // 1f0c → 1f04
    {u'Ἅ', u'ἅ'},      // 1f0d → 1f05
    {u'Ἆ', u'ἆ'},      // 1f0e → 1f06
    {u'Ἇ', u'ἇ'},      // 1f0f → 1f07
    {u'Ἐ', u'ἐ'},      // 1f18 → 1f10
    {u'Ἑ', u'ἑ'},      // 1f19 → 1f11
    {u'Ἒ', u'ἒ'},      // 1f1a → 1f12
    {u'Ἓ', u'ἓ'},      // 1f1b → 1f13
    {u'Ἔ', u'ἔ'},      // 1f1c → 1f14
    {u'Ἕ', u'ἕ'},      // 1f1d → 1f15
    {u'Ἠ', u'ἠ'},      // 1f28 → 1f20
    {u'Ἡ', u'ἡ'},      // 1f29 → 1f21
    {u'Ἢ', u'ἢ'},      // 1f2a → 1f22
    {u'Ἣ', u'ἣ'},      // 1f2b → 1f23
    {u'Ἤ', u'ἤ'},      // 1f2c → 1f24
    {u'Ἥ', u'ἥ'},      // 1f2d → 1f25
    {u'Ἦ', u'ἦ'},      // 1f2e → 1f26
    {u'Ἧ', u'ἧ'},      // 1f2f → 1f27
    {u'Ἰ', u'ἰ'},      // 1f38 → 1f30
    {u'Ἱ', u'ἱ'},      // 1f39 → 1f31
    {u'Ἲ', u'ἲ'},      // 1f3a → 1f32
    {u'Ἳ', u'ἳ'},      // 1f3b → 1f33
    {u'Ἴ', u'ἴ'},      // 1f3c → 1f34
    {u'Ἵ', u'ἵ'},      // 1f3d → 1f35
    {u'Ἶ', u'ἶ'},      // 1f3e → 1f36
    {u'Ἷ', u'ἷ'},      // 1f3f → 1f37
    {u'Ὀ', u'ὀ'},      // 1f48 → 1f40
    {u'Ὁ', u'ὁ'},      // 1f49 → 1f41
    {u'Ὂ', u'ὂ'},      // 1f4a → 1f42
    {u'Ὃ', u'ὃ'},      // 1f4b → 1f43
    {u'Ὄ', u'ὄ'},      // 1f4c → 1f44
    {u'Ὅ', u'ὅ'},      // 1f4d → 1f45
    {u'Ὑ', u'ὑ'},      // 1f59 → 1f51
    {u'Ὓ', u'ὓ'},      // 1f5b → 1f53
    {u'Ὕ', u'ὕ'},      // 1f5d → 1f55
    {u'Ὗ', u'ὗ'},      // 1f5f → 1f57
    {u'Ὠ', u'ὠ'},      // 1f68 → 1f60
    {u'Ὡ', u'ὡ'},      // 1f69 → 1f61
    {u'Ὢ', u'ὢ'},      // 1f6a → 1f62
    {u'Ὣ', u'ὣ'},      // 1f6b → 1f63
    {u'Ὤ', u'ὤ'},      // 1f6c → 1f64
    {u'Ὥ', u'ὥ'},      // 1f6d → 1f65
    {u'Ὦ', u'ὦ'},      // 1f6e → 1f66
    {u'Ὧ', u'ὧ'},      // 1f6f → 1f67
    {u'Ᾰ', u'ᾰ'},      // 1fb8 → 1fb0
    {u'Ᾱ', u'ᾱ'},      // 1fb9 → 1fb1
    {u'Ὰ', u'ὰ'},      // 1fba → 1f70
    {u'Ά', u'ά'},      // 1fbb → 1f71
    {u'Ὲ', u'ὲ'},      // 1fc8 → 1f72
    {u'Έ', u'έ'},      // 1fc9 → 1f73
    {u'Ὴ', u'ὴ'},      // 1fca → 1f74
    {u'Ή', u'ή'},      // 1fcb → 1f75
    {u'Ῐ', u'ῐ'},      // 1fd8 → 1fd0
    {u'Ῑ', u'ῑ'},      // 1fd9 → 1fd1
    {u'Ὶ', u'ὶ'},      // 1fda → 1f76
    {u'Ί', u'ί'},      // 1fdb → 1f77
    {u'Ῠ', u'ῠ'},      // 1fe8 → 1fe0
    {u'Ῡ', u'ῡ'},      // 1fe9 → 1fe1
    {u'Ὺ', u'ὺ'},      // 1fea → 1f7a
    {u'Ύ', u'ύ'},      // 1feb → 1f7b
    {u'Ῥ', u'ῥ'},      // 1fec → 1fe5
    {u'Ὸ', u'ὸ'},      // 1ff8 → 1f78
    {u'Ό', u'ό'},      // 1ff9 → 1f79
    {u'Ὼ', u'ὼ'},      // 1ffa → 1f7c
    {u'Ώ', u'ώ'},      // 1ffb → 1f7d
    {u'ℒ', u'ℓ'},      // 2112 → 2113
    {u'ℙ', u'ℼ'},      // 2119 → 213c
    {u'ℰ', u'ℯ'},      // 2130 → 212f
    {u'Ⅎ', u'ⅎ'},      // 2132 → 214e
    {u'ℾ', u'ℽ'},      // 213e → 213d
    {u'ℿ', u'ℼ'},      // 213f → 213c
    {u'ⅅ', u'ⅆ'},      // 2145 → 2146
    {u'Ⰰ', u'ⰰ'},      // 2c00 → 2c30
    {u'Ⰱ', u'ⰱ'},      // 2c01 → 2c31
    {u'Ⰲ', u'ⰲ'},      // 2c02 → 2c32
    {u'Ⰳ', u'ⰳ'},      // 2c03 → 2c33
    {u'Ⰴ', u'ⰴ'},      // 2c04 → 2c34
    {u'Ⰵ', u'ⰵ'},      // 2c05 → 2c35
    {u'Ⰶ', u'ⰶ'},      // 2c06 → 2c36
    {u'Ⰷ', u'ⰷ'},      // 2c07 → 2c37
    {u'Ⰸ', u'ⰸ'},      // 2c08 → 2c38
    {u'Ⰹ', u'ⰹ'},      // 2c09 → 2c39
    {u'Ⰺ', u'ⰺ'},      // 2c0a → 2c3a
    {u'Ⰻ', u'ⰹ'},      // 2c0b → 2c39
    {u'Ⰼ', u'ⰼ'},      // 2c0c → 2c3c
    {u'Ⰽ', u'ⰽ'},      // 2c0d → 2c3d
    {u'Ⰾ', u'ⰾ'},      // 2c0e → 2c3e
    {u'Ⰿ', u'ⰿ'},      // 2c0f → 2c3f
    {u'Ⱀ', u'ⱀ'},      // 2c10 → 2c40
    {u'Ⱁ', u'ⱁ'},      // 2c11 → 2c41
    {u'Ⱂ', u'ⱂ'},      // 2c12 → 2c42
    {u'Ⱃ', u'ⱃ'},      // 2c13 → 2c43
    {u'Ⱄ', u'ⱄ'},      // 2c14 → 2c44
    {u'Ⱅ', u'ⱅ'},      // 2c15 → 2c45
    {u'Ⱆ', u'ⱆ'},      // 2c16 → 2c46
    {u'Ⱇ', u'ⱇ'},      // 2c17 → 2c47
    {u'Ⱈ', u'ⱈ'},      // 2c18 → 2c48
    {u'Ⱉ', u'ⱉ'},      // 2c19 → 2c49
    {u'Ⱊ', u'ⱊ'},      // 2c1a → 2c4a
    {u'Ⱋ', u'ⱋ'},      // 2c1b → 2c4b
    {u'Ⱌ', u'ⱌ'},      // 2c1c → 2c4c
    {u'Ⱍ', u'ⱍ'},      // 2c1d → 2c4d
    {u'Ⱎ', u'ⱎ'},      // 2c1e → 2c4e
    {u'Ⱏ', u'ⱏ'},      // 2c1f → 2c4f
    {u'Ⱐ', u'ⱐ'},      // 2c20 → 2c50
    {u'Ⱑ', u'ⱑ'},      // 2c21 → 2c51
    {u'Ⱒ', u'ⱒ'},      // 2c22 → 2c52
    {u'Ⱓ', u'ⱓ'},      // 2c23 → 2c53
    {u'Ⱔ', u'ⱔ'},      // 2c24 → 2c54
    {u'Ⱕ', u'ⱕ'},      // 2c25 → 2c55
    {u'Ⱖ', u'ⱖ'},      // 2c26 → 2c56
    {u'Ⱗ', u'ⱗ'},      // 2c27 → 2c57
    {u'Ⱘ', u'ⱘ'},      // 2c28 → 2c58
    {u'Ⱙ', u'ⱙ'},      // 2c29 → 2c59
    {u'Ⱚ', u'ⱚ'},      // 2c2a → 2c5a
    {u'Ⱛ', u'ⱛ'},      // 2c2b → 2c5b
    {u'Ⱜ', u'ⱜ'},      // 2c2c → 2c5c
    {u'Ⱝ', u'ⱝ'},      // 2c2d → 2c5d
    {u'Ⱞ', u'ⱞ'},      // 2c2e → 2c5e
    {u'Ⱡ', u'ⱡ'},      // 2c60 → 2c61
    {u'Ɫ', u'ɫ'},      // 2c62 → 026b
    {u'Ᵽ', u'ᵽ'},      // 2c63 → 1d7d
    {u'Ɽ', u'ɽ'},      // 2c64 → 027d
    {u'Ⱨ', u'ⱨ'},      // 2c67 → 2c68
    {u'Ⱪ', u'ⱪ'},      // 2c69 → 2c6a
    {u'Ⱬ', u'ⱬ'},      // 2c6b → 2c6c
    {u'Ɑ', u'ɑ'},      // 2c6d → 0251
    {u'Ɱ', u'ɱ'},      // 2c6e → 0271
    {u'Ɐ', u'ɐ'},      // 2c6f → 0250
    {u'Ɒ', u'ɒ'},      // 2c70 → 0252
    {u'Ⱳ', u'ⱳ'},      // 2c72 → 2c73
    {u'Ⱶ', u'ⱶ'},      // 2c75 → 2c76
    {u'Ȿ', u'ȿ'},      // 2c7e → 023f
    {u'Ɀ', u'ɀ'},      // 2c7f → 0240
    {u'Ⲁ', u'ⲁ'},      // 2c80 → 2c81
    {u'Ⲃ', u'ⲃ'},      // 2c82 → 2c83
    {u'Ⲅ', u'ⲅ'},      // 2c84 → 2c85
    {u'Ⲇ', u'ⲇ'},      // 2c86 → 2c87
    {u'Ⲉ', u'ⲉ'},      // 2c88 → 2c89
    {u'Ⲋ', u'ⲋ'},      // 2c8a → 2c8b
    {u'Ⲍ', u'ⲍ'},      // 2c8c → 2c8d
    {u'Ⲏ', u'ⲏ'},      // 2c8e → 2c8f
    {u'Ⲑ', u'ⲑ'},      // 2c90 → 2c91
    {u'Ⲓ', u'ⲓ'},      // 2c92 → 2c93
    {u'Ⲕ', u'ⲕ'},      // 2c94 → 2c95
    {u'Ⲗ', u'ⲗ'},      // 2c96 → 2c97
    {u'Ⲙ', u'ⲙ'},      // 2c98 → 2c99
    {u'Ⲛ', u'ⲛ'},      // 2c9a → 2c9b
    {u'Ⲝ', u'ⲝ'},      // 2c9c → 2c9d
    {u'Ⲟ', u'ⲟ'},      // 2c9e → 2c9f
    {u'Ⲡ', u'ⲡ'},      // 2ca0 → 2ca1
    {u'Ⲣ', u'ⲣ'},      // 2ca2 → 2ca3
    {u'Ⲥ', u'ⲥ'},      // 2ca4 → 2ca5
    {u'Ⲧ', u'ⲧ'},      // 2ca6 → 2ca7
    {u'Ⲩ', u'ⲩ'},      // 2ca8 → 2ca9
    {u'Ⲫ', u'ⲫ'},      // 2caa → 2cab
    {u'Ⲭ', u'ⲭ'},      // 2cac → 2cad
    {u'Ⲯ', u'ⲯ'},      // 2cae → 2caf
    {u'Ⲱ', u'ⲱ'},      // 2cb0 → 2cb1
    {u'Ⲳ', u'ⲳ'},      // 2cb2 → 2cb3
    {u'Ⲵ', u'ⲵ'},      // 2cb4 → 2cb5
    {u'Ⲷ', u'ⲷ'},      // 2cb6 → 2cb7
    {u'Ⲹ', u'ⲹ'},      // 2cb8 → 2cb9
    {u'Ⲻ', u'ⲻ'},      // 2cba → 2cbb
    {u'Ⲽ', u'ⲽ'},      // 2cbc → 2cbd
    {u'Ⲿ', u'ⲿ'},      // 2cbe → 2cbf
    {u'Ⳁ', u'ⳁ'},      // 2cc0 → 2cc1
    {u'Ⳃ', u'ⳃ'},      // 2cc2 → 2cc3
    {u'Ⳅ', u'ⳅ'},      // 2cc4 → 2cc5
    {u'Ⳇ', u'ⳇ'},      // 2cc6 → 2cc7
    {u'Ⳉ', u'ⳉ'},      // 2cc8 → 2cc9
    {u'Ⳋ', u'ⳋ'},      // 2cca → 2ccb
    {u'Ⳍ', u'ⳍ'},      // 2ccc → 2ccd
    {u'Ⳏ', u'ⳏ'},      // 2cce → 2ccf
    {u'Ⳑ', u'ⳑ'},      // 2cd0 → 2cd1
    {u'Ⳓ', u'ⳓ'},      // 2cd2 → 2cd3
    {u'Ⳕ', u'ⳕ'},      // 2cd4 → 2cd5
    {u'Ⳗ', u'ⳗ'},      // 2cd6 → 2cd7
    {u'Ⳙ', u'ⳙ'},      // 2cd8 → 2cd9
    {u'Ⳛ', u'ⳛ'},      // 2cda → 2cdb
    {u'Ⳝ', u'ⳝ'},      // 2cdc → 2cdd
    {u'Ⳟ', u'ⳟ'},      // 2cde → 2cdf
    {u'Ⳡ', u'ⳡ'},      // 2ce0 → 2ce1
    {u'Ⳣ', u'ⳣ'},      // 2ce2 → 2ce3
    {u'Ⳬ', u'ⳬ'},      // 2ceb → 2cec
    {u'Ⳮ', u'ⳮ'},      // 2ced → 2cee
    {u'Ⳳ', u'ⳳ'},      // 2cf2 → 2cf3
    {u'Ꙁ', u'ꙁ'},      // a640 → a641
    {u'Ꙃ', u'ꙃ'},      // a642 → a643
    {u'Ꙅ', u'ꙅ'},      // a644 → a645
    {u'Ꙇ', u'ꙇ'},      // a646 → a647
    {u'Ꙉ', u'ꙉ'},      // a648 → a649
    {u'Ꙋ', u'ꙋ'},      // a64a → a64b
    {u'Ꙍ', u'ꙍ'},      // a64c → a64d
    {u'Ꙏ', u'ꙏ'},      // a64e → a64f
    {u'Ꙑ', u'ꙑ'},      // a650 → a651
    {u'Ꙓ', u'ꙓ'},      // a652 → a653
    {u'Ꙕ', u'ꙕ'},      // a654 → a655
    {u'Ꙗ', u'ꙗ'},      // a656 → a657
    {u'Ꙙ', u'ꙙ'},      // a658 → a659
    {u'Ꙛ', u'ꙛ'},      // a65a → a65b
    {u'Ꙝ', u'ꙝ'},      // a65c → a65d
    {u'Ꙟ', u'ꙟ'},      // a65e → a65f
    {u'Ꙡ', u'ꙡ'},      // a660 → a661
    {u'Ꙣ', u'ꙣ'},      // a662 → a663
    {u'Ꙥ', u'ꙥ'},      // a664 → a665
    {u'Ꙧ', u'ꙧ'},      // a666 → a667
    {u'Ꙩ', u'ꙩ'},      // a668 → a669
    {u'Ꙫ', u'ꙫ'},      // a66a → a66b
    {u'Ꙭ', u'ꙭ'},      // a66c → a66d
    {u'Ꚁ', u'ꚁ'},      // a680 → a681
    {u'Ꚃ', u'ꚃ'},      // a682 → a683
    {u'Ꚅ', u'ꚅ'},      // a684 → a685
    {u'Ꚇ', u'ꚇ'},      // a686 → a687
    {u'Ꚉ', u'ꚉ'},      // a688 → a689
    {u'Ꚋ', u'ꚋ'},      // a68a → a68b
    {u'Ꚍ', u'ꚍ'},      // a68c → a68d
    {u'Ꚏ', u'ꚏ'},      // a68e → a68f
    {u'Ꚑ', u'ꚑ'},      // a690 → a691
    {u'Ꚓ', u'ꚓ'},      // a692 → a693
    {u'Ꚕ', u'ꚕ'},      // a694 → a695
    {u'Ꚗ', u'ꚗ'},      // a696 → a697
    {u'Ꚙ', u'ꚙ'},  // a698 → a699
    {u'Ꚛ', u'ꚛ'},  // a69a → a69b
    {u'Ꜣ', u'ꜣ'},      // a722 → a723
    {u'Ꜥ', u'ꜥ'},      // a724 → a725
    {u'Ꜧ', u'ꜧ'},      // a726 → a727
    {u'Ꜩ', u'ꜩ'},      // a728 → a729
    {u'Ꜫ', u'ꜫ'},      // a72a → a72b
    {u'Ꜭ', u'ꜭ'},      // a72c → a72d
    {u'Ꜯ', u'ꜯ'},      // a72e → a72f
    {u'Ꜳ', u'ꜳ'},      // a732 → a733
    {u'Ꜵ', u'ꜵ'},      // a734 → a735
    {u'Ꜷ', u'ꜷ'},      // a736 → a737
    {u'Ꜹ', u'ꜹ'},      // a738 → a739
    {u'Ꜻ', u'ꜻ'},      // a73a → a73b
    {u'Ꜽ', u'ꜽ'},      // a73c → a73d
    {u'Ꜿ', u'ꜿ'},      // a73e → a73f
    {u'Ꝁ', u'ꝁ'},      // a740 → a741
    {u'Ꝃ', u'ꝃ'},      // a742 → a743
    {u'Ꝅ', u'ꝅ'},      // a744 → a745
    {u'Ꝇ', u'ꝇ'},      // a746 → a747
    {u'Ꝉ', u'ꝉ'},      // a748 → a749
    {u'Ꝋ', u'ꝋ'},      // a74a → a74b
    {u'Ꝍ', u'ꝍ'},      // a74c → a74d
    {u'Ꝏ', u'ꝏ'},      // a74e → a74f
    {u'Ꝑ', u'ꝑ'},      // a750 → a751
    {u'Ꝓ', u'ꝓ'},      // a752 → a753
    {u'Ꝕ', u'ꝕ'},      // a754 → a755
    {u'Ꝗ', u'ꝑ'},      // a756 → a751
    {u'Ꝙ', u'ꝙ'},      // a758 → a759
    {u'Ꝛ', u'ꝛ'},      // a75a → a75b
    {u'Ꝝ', u'ꝝ'},      // a75c → a75d
    {u'Ꝟ', u'ꝟ'},      // a75e → a75f
    {u'Ꝡ', u'ꝡ'},      // a760 → a761
    {u'Ꝣ', u'ꝣ'},      // a762 → a763
    {u'Ꝥ', u'ꝥ'},      // a764 → a765
    {u'Ꝧ', u'ꝑ'},      // a766 → a751
    {u'Ꝩ', u'ꝩ'},      // a768 → a769
    {u'Ꝫ', u'ꝫ'},      // a76a → a76b
    {u'Ꝭ', u'ꝭ'},      // a76c → a76d
    {u'Ꝯ', u'ꝯ'},      // a76e → a76f
    {u'Ꝺ', u'ꝺ'},      // a779 → a77a
    {u'Ꝼ', u'ꝼ'},      // a77b → a77c
    {u'Ᵹ', u'ᵹ'},      // a77d → 1d79
    {u'Ꝿ', u'ꝿ'},      // a77e → a77f
    {u'Ꞁ', u'ꞁ'},      // a780 → a781
    {u'Ꞃ', u'ꞃ'},      // a782 → a783
    {u'Ꞅ', u'ꞅ'},      // a784 → a785
    {u'Ꞇ', u'ꞇ'},      // a786 → a787
    {u'Ꞌ', u'ꞌ'},      // a78b → a78c
    {u'Ɥ', u'ɥ'},      // a78d → 0265
    {u'Ꞑ', u'ꞑ'},      // a790 → a791
    {u'Ꞓ', u'ꞓ'},      // a792 → a793
    {u'Ꞗ', u'ꞗ'},  // a796 → a797
    {u'Ꞙ', u'ꞙ'},  // a798 → a799
    {u'Ꞛ', u'ꞛ'},  // a79a → a79b
    {u'Ꞝ', u'ꞝ'},  // a79c → a79d
    {u'Ꞟ', u'ꞟ'},  // a79e → a79f
    {u'Ꞡ', u'ꞡ'},      // a7a0 → a7a1
    {u'Ꞣ', u'ꞣ'},      // a7a2 → a7a3
    {u'Ꞥ', u'ꞥ'},      // a7a4 → a7a5
    {u'Ꞧ', u'ꞧ'},      // a7a6 → a7a7
    {u'Ꞩ', u'ꞩ'},      // a7a8 → a7a9
    {u'Ɦ', u'ɦ'},      // a7aa → 0266
    {u'Ɜ', u'ɜ'},    // a7ab → 025c
    {u'Ɡ', u'ɡ'},    // a7ac → 0261
    {u'Ɬ', u'ɬ'},    // a7ad → 026c
    {u'Ʞ', u'ʞ'},    // a7b0 → 029e
    {u'Ʇ', u'ʇ'},    // a7b1 → 0287
    {u'Ʝ', u'ʝ'},    // a7b2 → 029d
    {u'Ꭓ', u'ꭓ'},  // a7b3 → ab53
    {u'Ꞵ', u'ꞵ'},  // a7b4 → a7b5
    {u'Ꞷ', u'ꞷ'},  // a7b6 → a7b7
    {u'Ꞹ', u'ꞹ'},  // a7b8 → a7b9
    {u'Ꞻ', u'ꞻ'},  // a7ba → a7bb
    {u'Ꞽ', u'ꞽ'},  // a7bc → a7bd
    {u'Ꞿ', u'ꞿ'},  // a7be → a7bf
    {u'Ꟃ', u'ꟃ'},  // a7c2 → a7c3
    {u'Ꞔ', u'ꞔ'},  // a7c4 → a794
    {u'Ʂ', u'ʂ'},    // a7c5 → 0282
    {u'Ᶎ', u'ᶎ'},    // a7c6 → 1d8e
    {u'Ａ', u'ａ'},    // ff21 → ff41
    {u'Ｂ', u'ｂ'},    // ff22 → ff42
    {u'Ｃ', u'ｃ'},    // ff23 → ff43
    {u'Ｄ', u'ｄ'},    // ff24 → ff44
    {u'Ｅ', u'ｅ'},    // ff25 → ff45
    {u'Ｆ', u'ｆ'},    // ff26 → ff46
    {u'Ｇ', u'ｇ'},    // ff27 → ff47
    {u'Ｈ', u'ｈ'},    // ff28 → ff48
    {u'Ｉ', u'ｉ'},    // ff29 → ff49
    {u'Ｊ', u'ｊ'},    // ff2a → ff4a
    {u'Ｋ', u'ｋ'},    // ff2b → ff4b
    {u'Ｌ', u'ｌ'},    // ff2c → ff4c
    {u'Ｍ', u'ｍ'},    // ff2d → ff4d
    {u'Ｎ', u'ｎ'},    // ff2e → ff4e
    {u'Ｏ', u'ｏ'},    // ff2f → ff4f
    {u'Ｐ', u'ｐ'},    // ff30 → ff50
    {u'Ｑ', u'ｑ'},    // ff31 → ff51
    {u'Ｒ', u'ｒ'},    // ff32 → ff52
    {u'Ｓ', u'ｓ'},    // ff33 → ff53
    {u'Ｔ', u'ｔ'},    // ff34 → ff54
    {u'Ｕ', u'ｕ'},    // ff35 → ff55
    {u'Ｖ', u'ｖ'},    // ff36 → ff56
    {u'Ｗ', u'ｗ'},    // ff37 → ff57
    {u'Ｘ', u'ｘ'},    // ff38 → ff58
    {u'Ｙ', u'ｙ'},    // ff39 → ff59
    {u'Ｚ', u'ｚ'},    // ff3a → ff5a
#endif
};

static const char32_t kUpperToLowerAstral[][2] = {
    {L'ℂ', L'𝕔'},        // 00002102 → 0001d554
    {L'ℋ', L'𝒽'},        // 0000210b → 0001d4bd
    {L'ℍ', L'𝕙'},        // 0000210d → 0001d559
    {L'ℐ', L'𝒾'},        // 00002110 → 0001d4be
    {L'ℕ', L'𝕟'},        // 00002115 → 0001d55f
    {L'ℚ', L'𝕢'},        // 0000211a → 0001d562
    {L'ℛ', L'𝓇'},        // 0000211b → 0001d4c7
    {L'ℝ', L'𝕣'},        // 0000211d → 0001d563
    {L'ℤ', L'𝕫'},        // 00002124 → 0001d56b
    {L'ℬ', L'𝒷'},        // 0000212c → 0001d4b7
    {L'ℱ', L'𝒻'},        // 00002131 → 0001d4bb
    {L'ℳ', L'𝓂'},        // 00002133 → 0001d4c2
    {L'𐐀', L'𐐨'},        // 00010400 → 00010428
    {L'𐐁', L'𐐩'},        // 00010401 → 00010429
    {L'𐐂', L'𐐪'},        // 00010402 → 0001042a
    {L'𐐃', L'𐐫'},        // 00010403 → 0001042b
    {L'𐐄', L'𐐬'},        // 00010404 → 0001042c
    {L'𐐅', L'𐐭'},        // 00010405 → 0001042d
    {L'𐐆', L'𐐮'},        // 00010406 → 0001042e
    {L'𐐇', L'𐐯'},        // 00010407 → 0001042f
    {L'𐐈', L'𐐰'},        // 00010408 → 00010430
    {L'𐐉', L'𐐱'},        // 00010409 → 00010431
    {L'𐐊', L'𐐲'},        // 0001040a → 00010432
    {L'𐐋', L'𐐳'},        // 0001040b → 00010433
    {L'𐐌', L'𐐴'},        // 0001040c → 00010434
    {L'𐐍', L'𐐵'},        // 0001040d → 00010435
    {L'𐐎', L'𐐶'},        // 0001040e → 00010436
    {L'𐐏', L'𐐷'},        // 0001040f → 00010437
    {L'𐐐', L'𐐸'},        // 00010410 → 00010438
    {L'𐐑', L'𐐹'},        // 00010411 → 00010439
    {L'𐐒', L'𐐺'},        // 00010412 → 0001043a
    {L'𐐓', L'𐐻'},        // 00010413 → 0001043b
    {L'𐐔', L'𐐼'},        // 00010414 → 0001043c
    {L'𐐕', L'𐐽'},        // 00010415 → 0001043d
    {L'𐐖', L'𐐾'},        // 00010416 → 0001043e
    {L'𐐗', L'𐐿'},        // 00010417 → 0001043f
    {L'𐐘', L'𐑀'},        // 00010418 → 00010440
    {L'𐐙', L'𐑁'},        // 00010419 → 00010441
    {L'𐐚', L'𐑂'},        // 0001041a → 00010442
    {L'𐐛', L'𐑃'},        // 0001041b → 00010443
    {L'𐐜', L'𐑄'},        // 0001041c → 00010444
    {L'𐐝', L'𐑅'},        // 0001041d → 00010445
    {L'𐐞', L'𐑆'},        // 0001041e → 00010446
    {L'𐐟', L'𐑇'},        // 0001041f → 00010447
    {L'𐐠', L'𐑈'},        // 00010420 → 00010448
    {L'𐐡', L'𐑉'},        // 00010421 → 00010449
    {L'𐐢', L'𐑊'},        // 00010422 → 0001044a
    {L'𐐣', L'𐑋'},        // 00010423 → 0001044b
    {L'𐐤', L'𐑌'},        // 00010424 → 0001044c
    {L'𐐥', L'𐑍'},        // 00010425 → 0001044d
    {L'𐐦', L'𐑎'},        // 00010426 → 0001044e
    {L'𐐧', L'𐑏'},        // 00010427 → 0001044f
    {L'𐒰', L'𐓘'},  // 000104b0 → 000104d8
    {L'𐒱', L'𐓙'},  // 000104b1 → 000104d9
    {L'𐒲', L'𐓚'},  // 000104b2 → 000104da
    {L'𐒳', L'𐓛'},  // 000104b3 → 000104db
    {L'𐒴', L'𐓜'},  // 000104b4 → 000104dc
    {L'𐒵', L'𐓝'},  // 000104b5 → 000104dd
    {L'𐒶', L'𐓞'},  // 000104b6 → 000104de
    {L'𐒷', L'𐓟'},  // 000104b7 → 000104df
    {L'𐒸', L'𐓠'},  // 000104b8 → 000104e0
    {L'𐒹', L'𐓡'},  // 000104b9 → 000104e1
    {L'𐒺', L'𐓢'},  // 000104ba → 000104e2
    {L'𐒻', L'𐓣'},  // 000104bb → 000104e3
    {L'𐒼', L'𐓤'},  // 000104bc → 000104e4
    {L'𐒽', L'𐓥'},  // 000104bd → 000104e5
    {L'𐒾', L'𐓦'},  // 000104be → 000104e6
    {L'𐒿', L'𐓧'},  // 000104bf → 000104e7
    {L'𐓀', L'𐓨'},  // 000104c0 → 000104e8
    {L'𐓁', L'𐓩'},  // 000104c1 → 000104e9
    {L'𐓂', L'𐓪'},  // 000104c2 → 000104ea
    {L'𐓃', L'𐓫'},  // 000104c3 → 000104eb
    {L'𐓄', L'𐓬'},  // 000104c4 → 000104ec
    {L'𐓅', L'𐓭'},  // 000104c5 → 000104ed
    {L'𐓆', L'𐓮'},  // 000104c6 → 000104ee
    {L'𐓇', L'𐓯'},  // 000104c7 → 000104ef
    {L'𐓈', L'𐓰'},  // 000104c8 → 000104f0
    {L'𐓉', L'𐓱'},  // 000104c9 → 000104f1
    {L'𐓊', L'𐓲'},  // 000104ca → 000104f2
    {L'𐓋', L'𐓳'},  // 000104cb → 000104f3
    {L'𐓌', L'𐓴'},  // 000104cc → 000104f4
    {L'𐓍', L'𐓵'},  // 000104cd → 000104f5
    {L'𐓎', L'𐓶'},  // 000104ce → 000104f6
    {L'𐓏', L'𐓷'},  // 000104cf → 000104f7
    {L'𐓐', L'𐓸'},  // 000104d0 → 000104f8
    {L'𐓑', L'𐓹'},  // 000104d1 → 000104f9
    {L'𐓒', L'𐓺'},  // 000104d2 → 000104fa
    {L'𐓓', L'𐓻'},  // 000104d3 → 000104fb
    {L'𐲀', L'𐳀'},  // 00010c80 → 00010cc0
    {L'𐲁', L'𐳁'},  // 00010c81 → 00010cc1
    {L'𐲂', L'𐳂'},  // 00010c82 → 00010cc2
    {L'𐲃', L'𐳃'},  // 00010c83 → 00010cc3
    {L'𐲄', L'𐳄'},  // 00010c84 → 00010cc4
    {L'𐲅', L'𐳅'},  // 00010c85 → 00010cc5
    {L'𐲆', L'𐳆'},  // 00010c86 → 00010cc6
    {L'𐲇', L'𐳇'},  // 00010c87 → 00010cc7
    {L'𐲈', L'𐳈'},  // 00010c88 → 00010cc8
    {L'𐲉', L'𐳂'},  // 00010c89 → 00010cc2
    {L'𐲊', L'𐳊'},  // 00010c8a → 00010cca
    {L'𐲋', L'𐳋'},  // 00010c8b → 00010ccb
    {L'𐲌', L'𐳌'},  // 00010c8c → 00010ccc
    {L'𐲍', L'𐳍'},  // 00010c8d → 00010ccd
    {L'𐲎', L'𐳎'},  // 00010c8e → 00010cce
    {L'𐲏', L'𐳏'},  // 00010c8f → 00010ccf
    {L'𐲐', L'𐳐'},  // 00010c90 → 00010cd0
    {L'𐲑', L'𐳑'},  // 00010c91 → 00010cd1
    {L'𐲒', L'𐳒'},  // 00010c92 → 00010cd2
    {L'𐲓', L'𐳓'},  // 00010c93 → 00010cd3
    {L'𐲔', L'𐳔'},  // 00010c94 → 00010cd4
    {L'𐲕', L'𐳕'},  // 00010c95 → 00010cd5
    {L'𐲖', L'𐳖'},  // 00010c96 → 00010cd6
    {L'𐲗', L'𐳗'},  // 00010c97 → 00010cd7
    {L'𐲘', L'𐳘'},  // 00010c98 → 00010cd8
    {L'𐲙', L'𐳅'},  // 00010c99 → 00010cc5
    {L'𐲚', L'𐳚'},  // 00010c9a → 00010cda
    {L'𐲛', L'𐳛'},  // 00010c9b → 00010cdb
    {L'𐲜', L'𐳜'},  // 00010c9c → 00010cdc
    {L'𐲝', L'𐳝'},  // 00010c9d → 00010cdd
    {L'𐲞', L'𐳞'},  // 00010c9e → 00010cde
    {L'𐲟', L'𐳟'},  // 00010c9f → 00010cdf
    {L'𐲠', L'𐳠'},  // 00010ca0 → 00010ce0
    {L'𐲡', L'𐳡'},  // 00010ca1 → 00010ce1
    {L'𐲢', L'𐳢'},  // 00010ca2 → 00010ce2
    {L'𐲣', L'𐳣'},  // 00010ca3 → 00010ce3
    {L'𐲤', L'𐳤'},  // 00010ca4 → 00010ce4
    {L'𐲥', L'𐳥'},  // 00010ca5 → 00010ce5
    {L'𐲦', L'𐳦'},  // 00010ca6 → 00010ce6
    {L'𐲧', L'𐳧'},  // 00010ca7 → 00010ce7
    {L'𐲨', L'𐳨'},  // 00010ca8 → 00010ce8
    {L'𐲩', L'𐳩'},  // 00010ca9 → 00010ce9
    {L'𐲪', L'𐳕'},  // 00010caa → 00010cd5
    {L'𐲫', L'𐳫'},  // 00010cab → 00010ceb
    {L'𐲬', L'𐳬'},  // 00010cac → 00010cec
    {L'𐲭', L'𐳭'},  // 00010cad → 00010ced
    {L'𐲮', L'𐳮'},  // 00010cae → 00010cee
    {L'𐲯', L'𐳯'},  // 00010caf → 00010cef
    {L'𐲰', L'𐳰'},  // 00010cb0 → 00010cf0
    {L'𐲱', L'𐳱'},  // 00010cb1 → 00010cf1
    {L'𐲲', L'𐳲'},  // 00010cb2 → 00010cf2
    {L'𑢠', L'𑣀'},  // 000118a0 → 000118c0
    {L'𑢡', L'𑣁'},  // 000118a1 → 000118c1
    {L'𑢢', L'𑣂'},  // 000118a2 → 000118c2
    {L'𑢣', L'𑣃'},  // 000118a3 → 000118c3
    {L'𑢤', L'𑣄'},  // 000118a4 → 000118c4
    {L'𑢥', L'𑣅'},  // 000118a5 → 000118c5
    {L'𑢦', L'𑣆'},  // 000118a6 → 000118c6
    {L'𑢧', L'𑣇'},  // 000118a7 → 000118c7
    {L'𑢨', L'𑣈'},  // 000118a8 → 000118c8
    {L'𑢩', L'𑣉'},  // 000118a9 → 000118c9
    {L'𑢪', L'𑣊'},  // 000118aa → 000118ca
    {L'𑢫', L'𑣋'},  // 000118ab → 000118cb
    {L'𑢬', L'𑣌'},  // 000118ac → 000118cc
    {L'𑢭', L'𑣍'},  // 000118ad → 000118cd
    {L'𑢮', L'𑣎'},  // 000118ae → 000118ce
    {L'𑢯', L'𑣏'},  // 000118af → 000118cf
    {L'𑢰', L'𑣐'},  // 000118b0 → 000118d0
    {L'𑢱', L'𑣑'},  // 000118b1 → 000118d1
    {L'𑢲', L'𑣒'},  // 000118b2 → 000118d2
    {L'𑢳', L'𑣓'},  // 000118b3 → 000118d3
    {L'𑢴', L'𑣔'},  // 000118b4 → 000118d4
    {L'𑢵', L'𑣕'},  // 000118b5 → 000118d5
    {L'𑢶', L'𑣖'},  // 000118b6 → 000118d6
    {L'𑢷', L'𑣗'},  // 000118b7 → 000118d7
    {L'𑢸', L'𑣘'},  // 000118b8 → 000118d8
    {L'𑢹', L'𑣙'},  // 000118b9 → 000118d9
    {L'𑢺', L'𑣚'},  // 000118ba → 000118da
    {L'𑢻', L'𑣛'},  // 000118bb → 000118db
    {L'𑢼', L'𑣜'},  // 000118bc → 000118dc
    {L'𑢽', L'𑣝'},  // 000118bd → 000118dd
    {L'𑢾', L'𑣞'},  // 000118be → 000118de
    {L'𑢿', L'𑣟'},  // 000118bf → 000118df
    {L'𖹀', L'𖹠'},  // 00016e40 → 00016e60
    {L'𖹁', L'𖹡'},  // 00016e41 → 00016e61
    {L'𖹂', L'𖹢'},  // 00016e42 → 00016e62
    {L'𖹃', L'𖹣'},  // 00016e43 → 00016e63
    {L'𖹄', L'𖹤'},  // 00016e44 → 00016e64
    {L'𖹅', L'𖹥'},  // 00016e45 → 00016e65
    {L'𖹆', L'𖹦'},  // 00016e46 → 00016e66
    {L'𖹇', L'𖹧'},  // 00016e47 → 00016e67
    {L'𖹈', L'𖹨'},  // 00016e48 → 00016e68
    {L'𖹉', L'𖹩'},  // 00016e49 → 00016e69
    {L'𖹊', L'𖹪'},  // 00016e4a → 00016e6a
    {L'𖹋', L'𖹫'},  // 00016e4b → 00016e6b
    {L'𖹌', L'𖹬'},  // 00016e4c → 00016e6c
    {L'𖹍', L'𖹭'},  // 00016e4d → 00016e6d
    {L'𖹎', L'𖹮'},  // 00016e4e → 00016e6e
    {L'𖹏', L'𖹯'},  // 00016e4f → 00016e6f
    {L'𖹐', L'𖹰'},  // 00016e50 → 00016e70
    {L'𖹑', L'𖹱'},  // 00016e51 → 00016e71
    {L'𖹒', L'𖹲'},  // 00016e52 → 00016e72
    {L'𖹓', L'𖹳'},  // 00016e53 → 00016e73
    {L'𖹔', L'𖹴'},  // 00016e54 → 00016e74
    {L'𖹕', L'𖹵'},  // 00016e55 → 00016e75
    {L'𖹖', L'𖹶'},  // 00016e56 → 00016e76
    {L'𖹗', L'𖹷'},  // 00016e57 → 00016e77
    {L'𖹘', L'𖹸'},  // 00016e58 → 00016e78
    {L'𖹙', L'𖹹'},  // 00016e59 → 00016e79
    {L'𖹚', L'𖹺'},  // 00016e5a → 00016e7a
    {L'𖹛', L'𖹻'},  // 00016e5b → 00016e7b
    {L'𖹜', L'𖹼'},  // 00016e5c → 00016e7c
    {L'𖹝', L'𖹽'},  // 00016e5d → 00016e7d
    {L'𖹞', L'𖹾'},  // 00016e5e → 00016e7e
    {L'𖹟', L'𖹿'},  // 00016e5f → 00016e7f
    {L'𝐀', L'𝐚'},        // 0001d400 → 0001d41a
    {L'𝐁', L'𝐛'},        // 0001d401 → 0001d41b
    {L'𝐂', L'𝐜'},        // 0001d402 → 0001d41c
    {L'𝐃', L'𝐝'},        // 0001d403 → 0001d41d
    {L'𝐄', L'𝐞'},        // 0001d404 → 0001d41e
    {L'𝐅', L'𝐟'},        // 0001d405 → 0001d41f
    {L'𝐆', L'𝐠'},        // 0001d406 → 0001d420
    {L'𝐇', L'𝐡'},        // 0001d407 → 0001d421
    {L'𝐈', L'𝐢'},        // 0001d408 → 0001d422
    {L'𝐉', L'𝐣'},        // 0001d409 → 0001d423
    {L'𝐊', L'𝐤'},        // 0001d40a → 0001d424
    {L'𝐋', L'𝐥'},        // 0001d40b → 0001d425
    {L'𝐌', L'𝐦'},        // 0001d40c → 0001d426
    {L'𝐍', L'𝐧'},        // 0001d40d → 0001d427
    {L'𝐎', L'𝐨'},        // 0001d40e → 0001d428
    {L'𝐏', L'𝐩'},        // 0001d40f → 0001d429
    {L'𝐐', L'𝐪'},        // 0001d410 → 0001d42a
    {L'𝐑', L'𝐫'},        // 0001d411 → 0001d42b
    {L'𝐒', L'𝐬'},        // 0001d412 → 0001d42c
    {L'𝐓', L'𝐭'},        // 0001d413 → 0001d42d
    {L'𝐔', L'𝐮'},        // 0001d414 → 0001d42e
    {L'𝐕', L'𝐯'},        // 0001d415 → 0001d42f
    {L'𝐖', L'𝐰'},        // 0001d416 → 0001d430
    {L'𝐗', L'𝐱'},        // 0001d417 → 0001d431
    {L'𝐘', L'𝐲'},        // 0001d418 → 0001d432
    {L'𝐙', L'𝐳'},        // 0001d419 → 0001d433
    {L'𝐴', L'𝑎'},        // 0001d434 → 0001d44e
    {L'𝐵', L'𝑏'},        // 0001d435 → 0001d44f
    {L'𝐶', L'𝑐'},        // 0001d436 → 0001d450
    {L'𝐷', L'𝑑'},        // 0001d437 → 0001d451
    {L'𝐸', L'𝑒'},        // 0001d438 → 0001d452
    {L'𝐹', L'𝑓'},        // 0001d439 → 0001d453
    {L'𝐺', L'𝑔'},        // 0001d43a → 0001d454
    {L'𝐼', L'𝑖'},        // 0001d43c → 0001d456
    {L'𝐽', L'𝑗'},        // 0001d43d → 0001d457
    {L'𝐾', L'𝑘'},        // 0001d43e → 0001d458
    {L'𝐿', L'𝑙'},        // 0001d43f → 0001d459
    {L'𝑀', L'𝑚'},        // 0001d440 → 0001d45a
    {L'𝑁', L'𝑛'},        // 0001d441 → 0001d45b
    {L'𝑂', L'𝑜'},        // 0001d442 → 0001d45c
    {L'𝑃', L'𝑝'},        // 0001d443 → 0001d45d
    {L'𝑄', L'𝑞'},        // 0001d444 → 0001d45e
    {L'𝑅', L'𝑟'},        // 0001d445 → 0001d45f
    {L'𝑆', L'𝑠'},        // 0001d446 → 0001d460
    {L'𝑇', L'𝑡'},        // 0001d447 → 0001d461
    {L'𝑈', L'𝑢'},        // 0001d448 → 0001d462
    {L'𝑉', L'𝑣'},        // 0001d449 → 0001d463
    {L'𝑊', L'𝑤'},        // 0001d44a → 0001d464
    {L'𝑋', L'𝑥'},        // 0001d44b → 0001d465
    {L'𝑌', L'𝑦'},        // 0001d44c → 0001d466
    {L'𝑍', L'𝑧'},        // 0001d44d → 0001d467
    {L'𝑨', L'𝒂'},        // 0001d468 → 0001d482
    {L'𝑩', L'𝒃'},        // 0001d469 → 0001d483
    {L'𝑪', L'𝒄'},        // 0001d46a → 0001d484
    {L'𝑫', L'𝒅'},        // 0001d46b → 0001d485
    {L'𝑬', L'𝒆'},        // 0001d46c → 0001d486
    {L'𝑭', L'𝒇'},        // 0001d46d → 0001d487
    {L'𝑮', L'𝒈'},        // 0001d46e → 0001d488
    {L'𝑯', L'𝒉'},        // 0001d46f → 0001d489
    {L'𝑰', L'𝒊'},        // 0001d470 → 0001d48a
    {L'𝑱', L'𝒋'},        // 0001d471 → 0001d48b
    {L'𝑲', L'𝒌'},        // 0001d472 → 0001d48c
    {L'𝑳', L'𝒍'},        // 0001d473 → 0001d48d
    {L'𝑴', L'𝒎'},        // 0001d474 → 0001d48e
    {L'𝑵', L'𝒏'},        // 0001d475 → 0001d48f
    {L'𝑶', L'𝒐'},        // 0001d476 → 0001d490
    {L'𝑷', L'𝒑'},        // 0001d477 → 0001d491
    {L'𝑸', L'𝒒'},        // 0001d478 → 0001d492
    {L'𝑹', L'𝒓'},        // 0001d479 → 0001d493
    {L'𝑺', L'𝒔'},        // 0001d47a → 0001d494
    {L'𝑻', L'𝒕'},        // 0001d47b → 0001d495
    {L'𝑼', L'𝒖'},        // 0001d47c → 0001d496
    {L'𝑽', L'𝒗'},        // 0001d47d → 0001d497
    {L'𝑾', L'𝒘'},        // 0001d47e → 0001d498
    {L'𝑿', L'𝒙'},        // 0001d47f → 0001d499
    {L'𝒀', L'𝒚'},        // 0001d480 → 0001d49a
    {L'𝒁', L'𝒛'},        // 0001d481 → 0001d49b
    {L'𝒜', L'𝒶'},        // 0001d49c → 0001d4b6
    {L'𝒞', L'𝒸'},        // 0001d49e → 0001d4b8
    {L'𝒟', L'𝒹'},        // 0001d49f → 0001d4b9
    {L'𝒥', L'𝒿'},        // 0001d4a5 → 0001d4bf
    {L'𝒦', L'𝓀'},        // 0001d4a6 → 0001d4c0
    {L'𝒩', L'𝓃'},        // 0001d4a9 → 0001d4c3
    {L'𝒫', L'𝓅'},        // 0001d4ab → 0001d4c5
    {L'𝒬', L'𝓆'},        // 0001d4ac → 0001d4c6
    {L'𝒮', L'𝓈'},        // 0001d4ae → 0001d4c8
    {L'𝒯', L'𝓉'},        // 0001d4af → 0001d4c9
    {L'𝒰', L'𝓊'},        // 0001d4b0 → 0001d4ca
    {L'𝒱', L'𝓋'},        // 0001d4b1 → 0001d4cb
    {L'𝒲', L'𝓌'},        // 0001d4b2 → 0001d4cc
    {L'𝒳', L'𝓍'},        // 0001d4b3 → 0001d4cd
    {L'𝒴', L'𝓎'},        // 0001d4b4 → 0001d4ce
    {L'𝒵', L'𝓏'},        // 0001d4b5 → 0001d4cf
    {L'𝓐', L'𝓪'},        // 0001d4d0 → 0001d4ea
    {L'𝓑', L'𝓫'},        // 0001d4d1 → 0001d4eb
    {L'𝓒', L'𝓬'},        // 0001d4d2 → 0001d4ec
    {L'𝓓', L'𝓭'},        // 0001d4d3 → 0001d4ed
    {L'𝓔', L'𝓮'},        // 0001d4d4 → 0001d4ee
    {L'𝓕', L'𝓯'},        // 0001d4d5 → 0001d4ef
    {L'𝓖', L'𝓰'},        // 0001d4d6 → 0001d4f0
    {L'𝓗', L'𝓱'},        // 0001d4d7 → 0001d4f1
    {L'𝓘', L'𝓲'},        // 0001d4d8 → 0001d4f2
    {L'𝓙', L'𝓳'},        // 0001d4d9 → 0001d4f3
    {L'𝓚', L'𝓴'},        // 0001d4da → 0001d4f4
    {L'𝓛', L'𝓵'},        // 0001d4db → 0001d4f5
    {L'𝓜', L'𝓶'},        // 0001d4dc → 0001d4f6
    {L'𝓝', L'𝓷'},        // 0001d4dd → 0001d4f7
    {L'𝓞', L'𝓸'},        // 0001d4de → 0001d4f8
    {L'𝓟', L'𝓹'},        // 0001d4df → 0001d4f9
    {L'𝓠', L'𝓺'},        // 0001d4e0 → 0001d4fa
    {L'𝓡', L'𝓻'},        // 0001d4e1 → 0001d4fb
    {L'𝓢', L'𝓼'},        // 0001d4e2 → 0001d4fc
    {L'𝓣', L'𝓽'},        // 0001d4e3 → 0001d4fd
    {L'𝓤', L'𝓾'},        // 0001d4e4 → 0001d4fe
    {L'𝓥', L'𝓿'},        // 0001d4e5 → 0001d4ff
    {L'𝓦', L'𝔀'},        // 0001d4e6 → 0001d500
    {L'𝓧', L'𝔁'},        // 0001d4e7 → 0001d501
    {L'𝓨', L'𝔂'},        // 0001d4e8 → 0001d502
    {L'𝓩', L'𝔃'},        // 0001d4e9 → 0001d503
    {L'𝔄', L'𝔞'},        // 0001d504 → 0001d51e
    {L'𝔅', L'𝔟'},        // 0001d505 → 0001d51f
    {L'𝔇', L'𝔡'},        // 0001d507 → 0001d521
    {L'𝔈', L'𝔢'},        // 0001d508 → 0001d522
    {L'𝔉', L'𝔣'},        // 0001d509 → 0001d523
    {L'𝔊', L'𝔤'},        // 0001d50a → 0001d524
    {L'𝔍', L'𝔧'},        // 0001d50d → 0001d527
    {L'𝔎', L'𝔨'},        // 0001d50e → 0001d528
    {L'𝔏', L'𝔩'},        // 0001d50f → 0001d529
    {L'𝔐', L'𝔪'},        // 0001d510 → 0001d52a
    {L'𝔑', L'𝔫'},        // 0001d511 → 0001d52b
    {L'𝔒', L'𝔬'},        // 0001d512 → 0001d52c
    {L'𝔓', L'𝔭'},        // 0001d513 → 0001d52d
    {L'𝔔', L'𝔮'},        // 0001d514 → 0001d52e
    {L'𝔖', L'𝔰'},        // 0001d516 → 0001d530
    {L'𝔗', L'𝔱'},        // 0001d517 → 0001d531
    {L'𝔘', L'𝔲'},        // 0001d518 → 0001d532
    {L'𝔙', L'𝔳'},        // 0001d519 → 0001d533
    {L'𝔚', L'𝔴'},        // 0001d51a → 0001d534
    {L'𝔛', L'𝔵'},        // 0001d51b → 0001d535
    {L'𝔜', L'𝔶'},        // 0001d51c → 0001d536
    {L'𝔸', L'𝕒'},        // 0001d538 → 0001d552
    {L'𝔹', L'𝕓'},        // 0001d539 → 0001d553
    {L'𝔻', L'𝕕'},        // 0001d53b → 0001d555
    {L'𝔼', L'𝕖'},        // 0001d53c → 0001d556
    {L'𝔽', L'𝕗'},        // 0001d53d → 0001d557
    {L'𝕀', L'𝕚'},        // 0001d540 → 0001d55a
    {L'𝕁', L'𝕛'},        // 0001d541 → 0001d55b
    {L'𝕂', L'𝕜'},        // 0001d542 → 0001d55c
    {L'𝕃', L'𝕝'},        // 0001d543 → 0001d55d
    {L'𝕄', L'𝕞'},        // 0001d544 → 0001d55e
    {L'𝕆', L'𝕠'},        // 0001d546 → 0001d560
    {L'𝕊', L'𝕤'},        // 0001d54a → 0001d564
    {L'𝕋', L'𝕥'},        // 0001d54b → 0001d565
    {L'𝕌', L'𝕦'},        // 0001d54c → 0001d566
    {L'𝕍', L'𝕧'},        // 0001d54d → 0001d567
    {L'𝕎', L'𝕨'},        // 0001d54e → 0001d568
    {L'𝕏', L'𝕩'},        // 0001d54f → 0001d569
    {L'𝕐', L'𝕪'},        // 0001d550 → 0001d56a
    {L'𝕬', L'𝖆'},        // 0001d56c → 0001d586
    {L'𝕭', L'𝖇'},        // 0001d56d → 0001d587
    {L'𝕮', L'𝖈'},        // 0001d56e → 0001d588
    {L'𝕯', L'𝖉'},        // 0001d56f → 0001d589
    {L'𝕰', L'𝖊'},        // 0001d570 → 0001d58a
    {L'𝕱', L'𝖋'},        // 0001d571 → 0001d58b
    {L'𝕲', L'𝖌'},        // 0001d572 → 0001d58c
    {L'𝕳', L'𝖍'},        // 0001d573 → 0001d58d
    {L'𝕴', L'𝖎'},        // 0001d574 → 0001d58e
    {L'𝕵', L'𝖏'},        // 0001d575 → 0001d58f
    {L'𝕶', L'𝖐'},        // 0001d576 → 0001d590
    {L'𝕷', L'𝖑'},        // 0001d577 → 0001d591
    {L'𝕸', L'𝖒'},        // 0001d578 → 0001d592
    {L'𝕹', L'𝖓'},        // 0001d579 → 0001d593
    {L'𝕺', L'𝖔'},        // 0001d57a → 0001d594
    {L'𝕻', L'𝖕'},        // 0001d57b → 0001d595
    {L'𝕼', L'𝖖'},        // 0001d57c → 0001d596
    {L'𝕽', L'𝖗'},        // 0001d57d → 0001d597
    {L'𝕾', L'𝖘'},        // 0001d57e → 0001d598
    {L'𝕿', L'𝖙'},        // 0001d57f → 0001d599
    {L'𝖀', L'𝖚'},        // 0001d580 → 0001d59a
    {L'𝖁', L'𝖛'},        // 0001d581 → 0001d59b
    {L'𝖂', L'𝖜'},        // 0001d582 → 0001d59c
    {L'𝖃', L'𝖝'},        // 0001d583 → 0001d59d
    {L'𝖄', L'𝖞'},        // 0001d584 → 0001d59e
    {L'𝖅', L'𝖟'},        // 0001d585 → 0001d59f
    {L'𝖠', L'𝖺'},        // 0001d5a0 → 0001d5ba
    {L'𝖡', L'𝖻'},        // 0001d5a1 → 0001d5bb
    {L'𝖢', L'𝖼'},        // 0001d5a2 → 0001d5bc
    {L'𝖣', L'𝖽'},        // 0001d5a3 → 0001d5bd
    {L'𝖤', L'𝖾'},        // 0001d5a4 → 0001d5be
    {L'𝖥', L'𝖿'},        // 0001d5a5 → 0001d5bf
    {L'𝖦', L'𝗀'},        // 0001d5a6 → 0001d5c0
    {L'𝖧', L'𝗁'},        // 0001d5a7 → 0001d5c1
    {L'𝖨', L'𝗂'},        // 0001d5a8 → 0001d5c2
    {L'𝖩', L'𝗃'},        // 0001d5a9 → 0001d5c3
    {L'𝖪', L'𝗄'},        // 0001d5aa → 0001d5c4
    {L'𝖫', L'𝗅'},        // 0001d5ab → 0001d5c5
    {L'𝖬', L'𝗆'},        // 0001d5ac → 0001d5c6
    {L'𝖭', L'𝗇'},        // 0001d5ad → 0001d5c7
    {L'𝖮', L'𝗈'},        // 0001d5ae → 0001d5c8
    {L'𝖯', L'𝗉'},        // 0001d5af → 0001d5c9
    {L'𝖰', L'𝗊'},        // 0001d5b0 → 0001d5ca
    {L'𝖱', L'𝗋'},        // 0001d5b1 → 0001d5cb
    {L'𝖲', L'𝗌'},        // 0001d5b2 → 0001d5cc
    {L'𝖳', L'𝗍'},        // 0001d5b3 → 0001d5cd
    {L'𝖴', L'𝗎'},        // 0001d5b4 → 0001d5ce
    {L'𝖵', L'𝗏'},        // 0001d5b5 → 0001d5cf
    {L'𝖶', L'𝗐'},        // 0001d5b6 → 0001d5d0
    {L'𝖷', L'𝗑'},        // 0001d5b7 → 0001d5d1
    {L'𝖸', L'𝗒'},        // 0001d5b8 → 0001d5d2
    {L'𝖹', L'𝗓'},        // 0001d5b9 → 0001d5d3
    {L'𝗔', L'𝗮'},        // 0001d5d4 → 0001d5ee
    {L'𝗕', L'𝗯'},        // 0001d5d5 → 0001d5ef
    {L'𝗖', L'𝗰'},        // 0001d5d6 → 0001d5f0
    {L'𝗗', L'𝗱'},        // 0001d5d7 → 0001d5f1
    {L'𝗘', L'𝗲'},        // 0001d5d8 → 0001d5f2
    {L'𝗙', L'𝗳'},        // 0001d5d9 → 0001d5f3
    {L'𝗚', L'𝗴'},        // 0001d5da → 0001d5f4
    {L'𝗛', L'𝗵'},        // 0001d5db → 0001d5f5
    {L'𝗜', L'𝗶'},        // 0001d5dc → 0001d5f6
    {L'𝗝', L'𝗷'},        // 0001d5dd → 0001d5f7
    {L'𝗞', L'𝗸'},        // 0001d5de → 0001d5f8
    {L'𝗟', L'𝗹'},        // 0001d5df → 0001d5f9
    {L'𝗠', L'𝗺'},        // 0001d5e0 → 0001d5fa
    {L'𝗡', L'𝗻'},        // 0001d5e1 → 0001d5fb
    {L'𝗢', L'𝗼'},        // 0001d5e2 → 0001d5fc
    {L'𝗣', L'𝗽'},        // 0001d5e3 → 0001d5fd
    {L'𝗤', L'𝗾'},        // 0001d5e4 → 0001d5fe
    {L'𝗥', L'𝗿'},        // 0001d5e5 → 0001d5ff
    {L'𝗦', L'𝘀'},        // 0001d5e6 → 0001d600
    {L'𝗧', L'𝘁'},        // 0001d5e7 → 0001d601
    {L'𝗨', L'𝘂'},        // 0001d5e8 → 0001d602
    {L'𝗩', L'𝘃'},        // 0001d5e9 → 0001d603
    {L'𝗪', L'𝘄'},        // 0001d5ea → 0001d604
    {L'𝗫', L'𝘅'},        // 0001d5eb → 0001d605
    {L'𝗬', L'𝘆'},        // 0001d5ec → 0001d606
    {L'𝗭', L'𝘇'},        // 0001d5ed → 0001d607
    {L'𝘈', L'𝘢'},        // 0001d608 → 0001d622
    {L'𝘉', L'𝘣'},        // 0001d609 → 0001d623
    {L'𝘊', L'𝘤'},        // 0001d60a → 0001d624
    {L'𝘋', L'𝘥'},        // 0001d60b → 0001d625
    {L'𝘌', L'𝘦'},        // 0001d60c → 0001d626
    {L'𝘍', L'𝘧'},        // 0001d60d → 0001d627
    {L'𝘎', L'𝘨'},        // 0001d60e → 0001d628
    {L'𝘏', L'𝘩'},        // 0001d60f → 0001d629
    {L'𝘐', L'𝘪'},        // 0001d610 → 0001d62a
    {L'𝘑', L'𝘫'},        // 0001d611 → 0001d62b
    {L'𝘒', L'𝘬'},        // 0001d612 → 0001d62c
    {L'𝘓', L'𝘭'},        // 0001d613 → 0001d62d
    {L'𝘔', L'𝘮'},        // 0001d614 → 0001d62e
    {L'𝘕', L'𝘯'},        // 0001d615 → 0001d62f
    {L'𝘖', L'𝘰'},        // 0001d616 → 0001d630
    {L'𝘗', L'𝘱'},        // 0001d617 → 0001d631
    {L'𝘘', L'𝘲'},        // 0001d618 → 0001d632
    {L'𝘙', L'𝘳'},        // 0001d619 → 0001d633
    {L'𝘚', L'𝘴'},        // 0001d61a → 0001d634
    {L'𝘛', L'𝘵'},        // 0001d61b → 0001d635
    {L'𝘜', L'𝘶'},        // 0001d61c → 0001d636
    {L'𝘝', L'𝘷'},        // 0001d61d → 0001d637
    {L'𝘞', L'𝘸'},        // 0001d61e → 0001d638
    {L'𝘟', L'𝘹'},        // 0001d61f → 0001d639
    {L'𝘠', L'𝘺'},        // 0001d620 → 0001d63a
    {L'𝘡', L'𝘻'},        // 0001d621 → 0001d63b
    {L'𝘼', L'𝒂'},        // 0001d63c → 0001d482
    {L'𝘽', L'𝒃'},        // 0001d63d → 0001d483
    {L'𝘾', L'𝒄'},        // 0001d63e → 0001d484
    {L'𝘿', L'𝒅'},        // 0001d63f → 0001d485
    {L'𝙀', L'𝒆'},        // 0001d640 → 0001d486
    {L'𝙁', L'𝒇'},        // 0001d641 → 0001d487
    {L'𝙂', L'𝒈'},        // 0001d642 → 0001d488
    {L'𝙃', L'𝒉'},        // 0001d643 → 0001d489
    {L'𝙄', L'𝒊'},        // 0001d644 → 0001d48a
    {L'𝙅', L'𝒋'},        // 0001d645 → 0001d48b
    {L'𝙆', L'𝒌'},        // 0001d646 → 0001d48c
    {L'𝙇', L'𝒍'},        // 0001d647 → 0001d48d
    {L'𝙈', L'𝒎'},        // 0001d648 → 0001d48e
    {L'𝙉', L'𝒏'},        // 0001d649 → 0001d48f
    {L'𝙊', L'𝒐'},        // 0001d64a → 0001d490
    {L'𝙋', L'𝒑'},        // 0001d64b → 0001d491
    {L'𝙌', L'𝒒'},        // 0001d64c → 0001d492
    {L'𝙍', L'𝒓'},        // 0001d64d → 0001d493
    {L'𝙎', L'𝒔'},        // 0001d64e → 0001d494
    {L'𝙏', L'𝒕'},        // 0001d64f → 0001d495
    {L'𝙐', L'𝒖'},        // 0001d650 → 0001d496
    {L'𝙑', L'𝒗'},        // 0001d651 → 0001d497
    {L'𝙒', L'𝒘'},        // 0001d652 → 0001d498
    {L'𝙓', L'𝒙'},        // 0001d653 → 0001d499
    {L'𝙔', L'𝒚'},        // 0001d654 → 0001d49a
    {L'𝙕', L'𝒛'},        // 0001d655 → 0001d49b
    {L'𝙰', L'𝚊'},        // 0001d670 → 0001d68a
    {L'𝙱', L'𝚋'},        // 0001d671 → 0001d68b
    {L'𝙲', L'𝚌'},        // 0001d672 → 0001d68c
    {L'𝙳', L'𝚍'},        // 0001d673 → 0001d68d
    {L'𝙴', L'𝚎'},        // 0001d674 → 0001d68e
    {L'𝙵', L'𝚏'},        // 0001d675 → 0001d68f
    {L'𝙶', L'𝚐'},        // 0001d676 → 0001d690
    {L'𝙷', L'𝚑'},        // 0001d677 → 0001d691
    {L'𝙸', L'𝚒'},        // 0001d678 → 0001d692
    {L'𝙹', L'𝚓'},        // 0001d679 → 0001d693
    {L'𝙺', L'𝚔'},        // 0001d67a → 0001d694
    {L'𝙻', L'𝚕'},        // 0001d67b → 0001d695
    {L'𝙼', L'𝚖'},        // 0001d67c → 0001d696
    {L'𝙽', L'𝚗'},        // 0001d67d → 0001d697
    {L'𝙾', L'𝚘'},        // 0001d67e → 0001d698
    {L'𝙿', L'𝚙'},        // 0001d67f → 0001d699
    {L'𝚀', L'𝚚'},        // 0001d680 → 0001d69a
    {L'𝚁', L'𝚛'},        // 0001d681 → 0001d69b
    {L'𝚂', L'𝚜'},        // 0001d682 → 0001d69c
    {L'𝚃', L'𝚝'},        // 0001d683 → 0001d69d
    {L'𝚄', L'𝚞'},        // 0001d684 → 0001d69e
    {L'𝚅', L'𝚟'},        // 0001d685 → 0001d69f
    {L'𝚆', L'𝚠'},        // 0001d686 → 0001d6a0
    {L'𝚇', L'𝚡'},        // 0001d687 → 0001d6a1
    {L'𝚈', L'𝚢'},        // 0001d688 → 0001d6a2
    {L'𝚉', L'𝚣'},        // 0001d689 → 0001d6a3
    {L'𝚨', L'𝛂'},        // 0001d6a8 → 0001d6c2
    {L'𝚩', L'𝛃'},        // 0001d6a9 → 0001d6c3
    {L'𝚪', L'𝛄'},        // 0001d6aa → 0001d6c4
    {L'𝚫', L'𝛅'},        // 0001d6ab → 0001d6c5
    {L'𝚬', L'𝛆'},        // 0001d6ac → 0001d6c6
    {L'𝚭', L'𝛇'},        // 0001d6ad → 0001d6c7
    {L'𝚮', L'𝛈'},        // 0001d6ae → 0001d6c8
    {L'𝚯', L'𝛉'},        // 0001d6af → 0001d6c9
    {L'𝚰', L'𝛊'},        // 0001d6b0 → 0001d6ca
    {L'𝚱', L'𝛋'},        // 0001d6b1 → 0001d6cb
    {L'𝚲', L'𝛌'},        // 0001d6b2 → 0001d6cc
    {L'𝚳', L'𝛍'},        // 0001d6b3 → 0001d6cd
    {L'𝚴', L'𝛎'},        // 0001d6b4 → 0001d6ce
    {L'𝚵', L'𝛏'},        // 0001d6b5 → 0001d6cf
    {L'𝚶', L'𝛐'},        // 0001d6b6 → 0001d6d0
    {L'𝚷', L'𝛑'},        // 0001d6b7 → 0001d6d1
    {L'𝚸', L'𝛒'},        // 0001d6b8 → 0001d6d2
    {L'𝚺', L'𝛔'},        // 0001d6ba → 0001d6d4
    {L'𝚻', L'𝛕'},        // 0001d6bb → 0001d6d5
    {L'𝚼', L'𝛖'},        // 0001d6bc → 0001d6d6
    {L'𝚽', L'𝛗'},        // 0001d6bd → 0001d6d7
    {L'𝚾', L'𝛘'},        // 0001d6be → 0001d6d8
    {L'𝚿', L'𝛙'},        // 0001d6bf → 0001d6d9
    {L'𝛀', L'𝛚'},        // 0001d6c0 → 0001d6da
    {L'𝛢', L'𝛼'},        // 0001d6e2 → 0001d6fc
    {L'𝛣', L'𝛽'},        // 0001d6e3 → 0001d6fd
    {L'𝛤', L'𝛾'},        // 0001d6e4 → 0001d6fe
    {L'𝛥', L'𝛿'},        // 0001d6e5 → 0001d6ff
    {L'𝛦', L'𝜀'},        // 0001d6e6 → 0001d700
    {L'𝛧', L'𝜁'},        // 0001d6e7 → 0001d701
    {L'𝛨', L'𝜂'},        // 0001d6e8 → 0001d702
    {L'𝛩', L'𝜃'},        // 0001d6e9 → 0001d703
    {L'𝛪', L'𝜄'},        // 0001d6ea → 0001d704
    {L'𝛫', L'𝜅'},        // 0001d6eb → 0001d705
    {L'𝛬', L'𝜆'},        // 0001d6ec → 0001d706
    {L'𝛭', L'𝜇'},        // 0001d6ed → 0001d707
    {L'𝛮', L'𝜈'},        // 0001d6ee → 0001d708
    {L'𝛯', L'𝜉'},        // 0001d6ef → 0001d709
    {L'𝛰', L'𝜊'},        // 0001d6f0 → 0001d70a
    {L'𝛱', L'𝜋'},        // 0001d6f1 → 0001d70b
    {L'𝛲', L'𝜌'},        // 0001d6f2 → 0001d70c
    {L'𝛳', L'𝜗'},        // 0001d6f3 → 0001d717
    {L'𝛴', L'𝜎'},        // 0001d6f4 → 0001d70e
    {L'𝛵', L'𝜏'},        // 0001d6f5 → 0001d70f
    {L'𝛶', L'𝜐'},        // 0001d6f6 → 0001d710
    {L'𝛷', L'𝜑'},        // 0001d6f7 → 0001d711
    {L'𝛸', L'𝜒'},        // 0001d6f8 → 0001d712
    {L'𝛹', L'𝜓'},        // 0001d6f9 → 0001d713
    {L'𝛺', L'𝜔'},        // 0001d6fa → 0001d714
    {L'𝜜', L'𝜶'},        // 0001d71c → 0001d736
    {L'𝜝', L'𝜷'},        // 0001d71d → 0001d737
    {L'𝜞', L'𝜸'},        // 0001d71e → 0001d738
    {L'𝜟', L'𝜹'},        // 0001d71f → 0001d739
    {L'𝜠', L'𝜺'},        // 0001d720 → 0001d73a
    {L'𝜡', L'𝜻'},        // 0001d721 → 0001d73b
    {L'𝜢', L'𝜼'},        // 0001d722 → 0001d73c
    {L'𝜣', L'𝜽'},        // 0001d723 → 0001d73d
    {L'𝜤', L'𝜾'},        // 0001d724 → 0001d73e
    {L'𝜥', L'𝜿'},        // 0001d725 → 0001d73f
    {L'𝜦', L'𝝀'},        // 0001d726 → 0001d740
    {L'𝜧', L'𝝁'},        // 0001d727 → 0001d741
    {L'𝜨', L'𝝂'},        // 0001d728 → 0001d742
    {L'𝜩', L'𝝃'},        // 0001d729 → 0001d743
    {L'𝜪', L'𝝄'},        // 0001d72a → 0001d744
    {L'𝜫', L'𝝅'},        // 0001d72b → 0001d745
    {L'𝜬', L'𝝆'},        // 0001d72c → 0001d746
    {L'𝜭', L'𝝑'},        // 0001d72d → 0001d751
    {L'𝜮', L'𝝈'},        // 0001d72e → 0001d748
    {L'𝜯', L'𝝉'},        // 0001d72f → 0001d749
    {L'𝜰', L'𝝊'},        // 0001d730 → 0001d74a
    {L'𝜱', L'𝝋'},        // 0001d731 → 0001d74b
    {L'𝜲', L'𝝌'},        // 0001d732 → 0001d74c
    {L'𝜳', L'𝝍'},        // 0001d733 → 0001d74d
    {L'𝜴', L'𝝎'},        // 0001d734 → 0001d74e
    {L'𝝖', L'𝝰'},        // 0001d756 → 0001d770
    {L'𝝗', L'𝝱'},        // 0001d757 → 0001d771
    {L'𝝘', L'𝝲'},        // 0001d758 → 0001d772
    {L'𝝙', L'𝝳'},        // 0001d759 → 0001d773
    {L'𝝚', L'𝝴'},        // 0001d75a → 0001d774
    {L'𝝛', L'𝝵'},        // 0001d75b → 0001d775
    {L'𝝜', L'𝝶'},        // 0001d75c → 0001d776
    {L'𝝝', L'𝝷'},        // 0001d75d → 0001d777
    {L'𝝞', L'𝝸'},        // 0001d75e → 0001d778
    {L'𝝟', L'𝝹'},        // 0001d75f → 0001d779
    {L'𝝠', L'𝝺'},        // 0001d760 → 0001d77a
    {L'𝝡', L'𝝻'},        // 0001d761 → 0001d77b
    {L'𝝢', L'𝝼'},        // 0001d762 → 0001d77c
    {L'𝝣', L'𝝽'},        // 0001d763 → 0001d77d
    {L'𝝤', L'𝝾'},        // 0001d764 → 0001d77e
    {L'𝝥', L'𝝿'},        // 0001d765 → 0001d77f
    {L'𝝦', L'𝞀'},        // 0001d766 → 0001d780
    {L'𝝧', L'𝟅'},        // 0001d767 → 0001d7c5
    {L'𝝨', L'𝞂'},        // 0001d768 → 0001d782
    {L'𝝩', L'𝞃'},        // 0001d769 → 0001d783
    {L'𝝪', L'𝞄'},        // 0001d76a → 0001d784
    {L'𝝫', L'𝞅'},        // 0001d76b → 0001d785
    {L'𝝬', L'𝞆'},        // 0001d76c → 0001d786
    {L'𝝭', L'𝞇'},        // 0001d76d → 0001d787
    {L'𝝮', L'𝞈'},        // 0001d76e → 0001d788
    {L'𝞐', L'𝜶'},        // 0001d790 → 0001d736
    {L'𝞑', L'𝜷'},        // 0001d791 → 0001d737
    {L'𝞒', L'𝜸'},        // 0001d792 → 0001d738
    {L'𝞓', L'𝜹'},        // 0001d793 → 0001d739
    {L'𝞔', L'𝜺'},        // 0001d794 → 0001d73a
    {L'𝞕', L'𝜻'},        // 0001d795 → 0001d73b
    {L'𝞖', L'𝜼'},        // 0001d796 → 0001d73c
    {L'𝞗', L'𝜽'},        // 0001d797 → 0001d73d
    {L'𝞘', L'𝜾'},        // 0001d798 → 0001d73e
    {L'𝞙', L'𝜿'},        // 0001d799 → 0001d73f
    {L'𝞚', L'𝝀'},        // 0001d79a → 0001d740
    {L'𝞛', L'𝝁'},        // 0001d79b → 0001d741
    {L'𝞜', L'𝝂'},        // 0001d79c → 0001d742
    {L'𝞝', L'𝝃'},        // 0001d79d → 0001d743
    {L'𝞞', L'𝝄'},        // 0001d79e → 0001d744
    {L'𝞟', L'𝝅'},        // 0001d79f → 0001d745
    {L'𝞠', L'𝝆'},        // 0001d7a0 → 0001d746
    {L'𝞡', L'𝟅'},        // 0001d7a1 → 0001d7c5
    {L'𝞢', L'𝝈'},        // 0001d7a2 → 0001d748
    {L'𝞣', L'𝝉'},        // 0001d7a3 → 0001d749
    {L'𝞤', L'𝝊'},        // 0001d7a4 → 0001d74a
    {L'𝞥', L'𝝋'},        // 0001d7a5 → 0001d74b
    {L'𝞦', L'𝝌'},        // 0001d7a6 → 0001d74c
    {L'𝞧', L'𝝍'},        // 0001d7a7 → 0001d74d
    {L'𝞨', L'𝝎'},        // 0001d7a8 → 0001d74e
    {L'𝟊', L'𝟋'},        // 0001d7ca → 0001d7cb
    {L'𞤀', L'𞤢'},  // 0001e900 → 0001e922
    {L'𞤁', L'𞤣'},  // 0001e901 → 0001e923
    {L'𞤂', L'𞤤'},  // 0001e902 → 0001e924
    {L'𞤃', L'𞤥'},  // 0001e903 → 0001e925
    {L'𞤄', L'𞤦'},  // 0001e904 → 0001e926
    {L'𞤅', L'𞤧'},  // 0001e905 → 0001e927
    {L'𞤆', L'𞤨'},  // 0001e906 → 0001e928
    {L'𞤇', L'𞤩'},  // 0001e907 → 0001e929
    {L'𞤈', L'𞤪'},  // 0001e908 → 0001e92a
    {L'𞤉', L'𞤫'},  // 0001e909 → 0001e92b
    {L'𞤊', L'𞤬'},  // 0001e90a → 0001e92c
    {L'𞤋', L'𞤭'},  // 0001e90b → 0001e92d
    {L'𞤌', L'𞤮'},  // 0001e90c → 0001e92e
    {L'𞤍', L'𞤯'},  // 0001e90d → 0001e92f
    {L'𞤎', L'𞤰'},  // 0001e90e → 0001e930
    {L'𞤏', L'𞤱'},  // 0001e90f → 0001e931
    {L'𞤐', L'𞤲'},  // 0001e910 → 0001e932
    {L'𞤑', L'𞤳'},  // 0001e911 → 0001e933
    {L'𞤒', L'𞤴'},  // 0001e912 → 0001e934
    {L'𞤓', L'𞤵'},  // 0001e913 → 0001e935
    {L'𞤔', L'𞤶'},  // 0001e914 → 0001e936
    {L'𞤕', L'𞤷'},  // 0001e915 → 0001e937
    {L'𞤖', L'𞤸'},  // 0001e916 → 0001e938
    {L'𞤗', L'𞤹'},  // 0001e917 → 0001e939
    {L'𞤘', L'𞤺'},  // 0001e918 → 0001e93a
    {L'𞤙', L'𞤻'},  // 0001e919 → 0001e93b
    {L'𞤚', L'𞤼'},  // 0001e91a → 0001e93c
    {L'𞤛', L'𞤽'},  // 0001e91b → 0001e93d
    {L'𞤜', L'𞤾'},  // 0001e91c → 0001e93e
    {L'𞤝', L'𞤿'},  // 0001e91d → 0001e93f
    {L'𞤞', L'𞥀'},  // 0001e91e → 0001e940
    {L'𞤟', L'𞥁'},  // 0001e91f → 0001e941
    {L'𞤠', L'𞥂'},  // 0001e920 → 0001e942
    {L'𞤡', L'𞥃'},  // 0001e921 → 0001e943
};

/**
 * Converts wide character to lower case.
 */
wint_t towlower(wint_t c) {
  int m, l, r;
  if (isascii(c)) return tolower(c);
  if (!IsTiny()) {
    l = 0;
    r = ARRAYLEN(kUpperToLowerAstral) - 1;
    while (l <= r) {
      m = (l + r) >> 1;
      if (kUpperToLowerAstral[m][0] < c) {
        l = m + 1;
      } else if (kUpperToLowerAstral[m][0] > c) {
        r = m - 1;
      } else {
        return kUpperToLowerAstral[m][1];
      }
    }
  }
  l = 0;
  r = ARRAYLEN(kUpperToLower) - 1;
  while (l <= r) {
    m = (l + r) >> 1;
    if (kUpperToLower[m][0] < c) {
      l = m + 1;
    } else if (kUpperToLower[m][0] > c) {
      r = m - 1;
    } else {
      return kUpperToLower[m][1];
    }
  }
  return c;
}
