/*-*- mode:c;indent-tabs-mode:nil;c-basic-offset:2;tab-width:8;coding:utf-8 -*-│
│vi: set net ft=c ts=2 sts=2 sw=2 fenc=utf-8                                :vi│
╞══════════════════════════════════════════════════════════════════════════════╡
│ Copyright 2020 Justine Alexandra Roberts Tunney                              │
│                                                                              │
│ Permission to use, copy, modify, and/or distribute this software for         │
│ any purpose with or without fee is hereby granted, provided that the         │
│ above copyright notice and this permission notice appear in all copies.      │
│                                                                              │
│ THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL                │
│ WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED                │
│ WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE             │
│ AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL         │
│ DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR        │
│ PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER               │
│ TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR             │
│ PERFORMANCE OF THIS SOFTWARE.                                                │
╚─────────────────────────────────────────────────────────────────────────────*/
#include "libc/calls/internal.h"
#include "libc/calls/ntspawn.h"
#include "libc/mem/alloca.h"
#include "libc/nt/accounting.h"
#include "libc/nt/enum/startf.h"
#include "libc/nt/enum/status.h"
#include "libc/nt/runtime.h"
#include "libc/nt/struct/processinformation.h"
#include "libc/nt/struct/startupinfo.h"
#include "libc/nt/synchronization.h"
#include "libc/str/str.h"
#include "libc/sysv/consts/o.h"

textwindows int sys_execve_nt(const char *program, char *const argv[],
                          char *const envp[]) {
  int rc;
  size_t i;
  uint32_t dwExitCode;
  struct NtStartupInfo startinfo;
  struct NtProcessInformation procinfo;
  memset(&startinfo, 0, sizeof(startinfo));
  startinfo.cb = sizeof(struct NtStartupInfo);
  startinfo.dwFlags = kNtStartfUsestdhandles;
  startinfo.hStdInput = g_fds.p[0].handle;
  startinfo.hStdOutput = g_fds.p[1].handle;
  startinfo.hStdError = g_fds.p[2].handle;
  for (i = 2; i < g_fds.n; ++i) {
    if (g_fds.p[i].kind != kFdEmpty && (g_fds.p[i].flags & O_CLOEXEC)) {
      close(i);
    }
  }
  rc = ntspawn(program, argv, envp, NULL, NULL, NULL, true, 0, NULL, &startinfo,
               &procinfo);
  if (rc == -1) return -1;
  CloseHandle(procinfo.hThread);
  do {
    WaitForSingleObject(procinfo.hProcess, -1);
    dwExitCode = kNtStillActive;
    GetExitCodeProcess(procinfo.hProcess, &dwExitCode);
  } while (dwExitCode == kNtStillActive);
  CloseHandle(procinfo.hProcess);
  ExitProcess(dwExitCode);
}
